/*
 * $Id: beacon.h,v 1.2 2015/01/09 22:09:49 charlesetsmith Exp $
 */

/*

 Copyright (c) 2011, Charles Smith
 All rights reserved.

 Redistribution and use in source and binary forms, with or without modification,
 are permitted provided that the following conditions are met:

    * Redistributions of source code must retain the above copyright notice, this
      list of conditions and the following disclaimer.
    * Redistributions in binary form must reproduce the above copyright notice, this
      list of conditions and the following disclaimer in the documentation and/or
      other materials provided with the distribution.
    * Neither the name of Vallona Networks nor the names of its contributors
      may be used to endorse or promote products derived from this software without
      specific prior written permission.

 THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND
 ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
 IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT,
 INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING,
 BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
 LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE
 OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED
 OF THE POSSIBILITY OF SUCH DAMAGE.

 */

#ifndef _BEACON_H
#define _BEACON_H

#include <iostream>
#include <cstring>
#include <string>
using namespace std;

#include "saratoga.h"
#include "sarflags.h"
#include "screen.h"
#include "timestamp.h"
#include "frame.h"
#include "dirent.h"
#include "ip.h"

namespace saratoga
{
/*
 **********************************************************************
 * BEACON
 **********************************************************************
 */
 
class beacon : public frame
{
private:
	flag_t		_flags; // Saratoga flags
	offset_t	_freespace; // Advertised freespace
	string		_eid; // The EID advertised
public:
	// This is how we assemble a local beacon
	// To get it ready for transmission
	// Create a beacon with no freespace identified
	beacon( enum f_descriptor,
		enum f_streams,
		enum f_txwilling,
		enum f_rxwilling,
		string);		// The EID 

	// This is how we assemble a local beacon
	// To get it ready for transmission
	// Create a beacon with freespace identified
	beacon( enum f_descriptor,
		enum f_streams,
		enum f_txwilling,
		enum f_rxwilling,
		fsinfo *,		// The Freespace
		string);		// The EID

	// We have received a remote frame that is a BEACON
	// Get it into a beacon class
	beacon(char	*,	// The Frames Payload
		size_t);	// Length of the payload

	~beacon() {
		delete[] _payload; // Base class variable
		_paylen = 0; // Base class variable
		_badframe = true; // We are not a valid frame
		_flags = 0; // Beacon variable
		_freespace = 0; // Beacon variable
		_eid.clear(); // Beacon variable
	};

	// Yes I am a beacon
	f_frametype type() { return F_FRAMETYPE_BEACON; };

	// What is the beacon eid
	string eid() { return _eid; };

	// What is the beacons advertised freespace
	offset_t	freespace() { return _freespace; };

	// What are the beacons flags
	flag_t	flags() { return _flags; };

	// These and only these flags can be changed during a session

	// Set tx or off return willing on,off,capable
	enum f_txwilling	txwilling(enum f_txwilling); // Set
	enum f_txwilling	txwilling(); // What is it ?

	// Set rx or off return willing on,off,capable
	enum f_rxwilling	rxwilling(enum f_rxwilling); // Set
	enum f_rxwilling	rxwilling(); // What is it ?

	// Transmit a beacon to a socket
	ssize_t tx(sarnet::udp *sock) {
		return(sock->tx(_payload, _paylen));
	};

	// Receive a beacon
	ssize_t rx() {
		string s = "BEACON RX:";
		s += this->print();
		cout << s << endl;
		return(-1);
	};

	// Print out the beacon details
	string	printflags();
	string	print();
};

} // Namespace saratoga

#endif // _BEACON_H

/*
 * $Log: beacon.h,v $
 * Revision 1.2  2015/01/09 22:09:49  charlesetsmith
 * Moved all Screen output to use curses. There are two windows, the top is status
 * and error messages. THe bottom is command line interaction. Also created
 * separate name spaces for curses (sarwin) and ip (sarip) so they can be
 * independant of the base saratoga code set. Fixed error in SConstruct
 * so that checksums now compiles properly and the checksums SConstruct
 * g++ options are same as the base code set. Huge number of changes but
 * the code set now operational. Remeber you have to "sudo ./saratoga"
 * as multicast requires you to be root to run.
 *
 * Revision 1.1.1.1  2014/10/15 11:55:35  lloydwood
 * Creating saratoga-vallona-dev.
 *
 * This is the Saratoga transfer protocol Vallona implementation,
 * under development by Charles Smith.
 *
 * Revision 1.11  2014/10/04 03:04:36  chas
 * Just too much to explain all. We are moving to using ncurses for terminal I/O
 * We have the basis for the main saratoga loop going
 * Beacon NEARLY works
 * Lots of command line additions (hence the need to get ncurses going. THere
 * is just WAY TOO MUCH screen stuff/debug going on. So Command line is
 * in the bottom window and Error/Status Outputs are in the top window.
 *
 * Revision 1.10  2013/10/04 02:04:18  chas
 * Finally fixed the damn issues with METADATA. Problem was that I was not
 * updating the Frame copy of the metadata. i.e. I needed to copy
 * to _payload and _paylen in the creator for metadata.
 *
 * Revision 1.9  2013/06/06 23:47:04  chas
 * Sorted out calculations of the available freespace so beacon id now TOTALLY DONE. Tidied up the allocator accordingly so now you only have to pass a pointer to the freespace and it sets the freespace descriptor up accordingly. So you can create beacons without freesoace or with in the allocator.
 *
 * Revision 1.8  2013/06/05 01:44:20  chas
 * Sorted out frame's
 *
 * Revision 1.7  2013/06/01 07:32:40  chas
 * Some issues with frame virtualisation. Perhaps this was a bad idea pulling the flags into the frame class... Ill think it over with a couple of scotches (single malt of course).
 *
 * Revision 1.6  2013/03/28 08:22:36  chas
 * Found an error in flags. Some wrong bit shifting for data flags
 * Progressing along with data class. And making printf's a bit more
 * portable with PRIuXX's for _session and _offset types
 *
 * Revision 1.5  2013/03/27 11:09:33  chas
 * Working on Data polymorph frame type now. Setting and getting flags
 * applicable to data and the optional timestamp. Some more thinking
 * on how I stamp the timestamp. THink it is best done within the
 * class constructor with an optional data.timestamp() to update it to
 * now when needed.
 *
 * Revision 1.4  2013/03/24 22:40:32  chas
 * Added flag changes to beacon. Will need to add more for other flags
 * and add the appropriate functions to other frame types as well.
 * I'll set up this in frame as well as virtual function calls.
 * This will ensure that only the appropriate flags are set for the approriate frame type.
 *
 * Revision 1.3  2013/03/24 22:01:13  chas
 * Added in beacon.h after splitting it out from frame.h
 * Makes things a lot more readable
 *
 */
