/*
 * $Id: cli.h,v 1.2 2015/01/09 22:09:49 charlesetsmith Exp $
 */

// Command Line Interpretor for saratoga

#ifndef _CLI_H
#define _CLI_H

#include <iostream>
#include <cstring>
#include <string>
#include <vector>
#include <list>
#include <sys/types.h>
#include "ip.h"
#include "sarflags.h"
#include "timestamp.h"

using namespace std;

namespace saratoga
{

// CLI Interface for exit
class cli_exit
{
private:
	int	_flag; // -1, 0, 1, 2
public:
	cli_exit() {
		_flag = -1;
	};
	
	~cli_exit() {
		_flag = -1;
	};
	void	flag(int x) { _flag = x; };
	int	flag() { return(_flag); };
};

// CLI Interface for beacon
class cli_beacon
{
private:
	bool	_v4flag; // broadcast v4 beacons
	bool	_v6flag; // broadcast v6 beacons
	int64_t	_secs; // send a beacon every x secs
	timestamp	*_timer; // time left to run
	std::list<sarnet::ip>	_ip; // IP Address's to send beacon to
public:
	cli_beacon() {
		_v4flag = false;
		_v6flag = false;
		_secs = 0;
		_timer = new timestamp();
	};
	
	~cli_beacon() {
		_v4flag = false;
		_v6flag = false;
		_secs = 0;
		delete _timer;
		this->zap();
	};

	void zap() 
	{ 
		while (! _ip.empty()) 
		{
			// delete _ip.front();
			_ip.pop_front(); 
		}
	};

	void	v4flag(bool x) { _v4flag = x; };
	bool	v4flag() { return(_v4flag); };
	void	v6flag(bool x) { _v6flag = x; };
	bool	v6flag() { return(_v6flag); };
	void	secs(size_t x) {_secs = x; };
	int64_t	secs() {return(_secs); };
	void	ipadd(string s) { this->iprm(s); _ip.push_back(s); };
	void	iprm(string s) 
	{
		sarnet::ip	tmp(s); 
		for (std::list<sarnet::ip>::iterator i = _ip.begin(); i != _ip.end(); i++)
		{
			if (*i == tmp)
			{
				_ip.remove(*i);
				break;
			}
		}
	};
	std::list<sarnet::ip>::iterator begin() { return(_ip.begin()); };
	std::list<sarnet::ip>::iterator end() { return(_ip.end()); };

	bool	canrun() { return (this->v4flag() || this->v6flag() || this->_ip.empty() == 0); };
	bool	timerexpired();

	timestamp*	timer() { return _timer; };

	void	settimer(const timestamp &tim) {
		delete _timer;
		_timer = new timestamp(tim);
	};

	string print();
	void execute(); // Run the beacon
};

class cli_bundles
{
private:
	bool	_flag; // Do we do bundles
public:
	cli_bundles() {
		_flag = false;
	};

	~cli_bundles() {
		_flag = false;
	};
	void	flag(bool x) { _flag = x; };
	bool	flag() { return(_flag); };
	string print();
};

class cli_checksum
{
private:
	enum f_csum _flag; // What type of checksums
				// F_CSUM_NONE, CRC32, MD5, SHA1
public:
	cli_checksum() {
		_flag = F_CSUM_NONE;
	};
	~cli_checksum() {
		_flag = F_CSUM_NONE;
	};
	void	flag(enum f_csum x) { _flag = x; };
	enum f_csum	flag() { return(_flag); };
	string print();
};

class cli_debug
{
private:
	int	_level; // 0-9
public:
	cli_debug() { 
		_level = 0;
	};
	~cli_debug() {
		_level = 0;
	};
	void	level(int lev) { 
		if (lev >= 0 && lev <= 9) 
			_level = lev; 
		else 
			_level = 0; };
	int	level() { return _level; };
	string print();
};

class cli_descriptor
{
private:
	enum f_descriptor _flag; // What type of descriptor
				// F_DESCRIPTOR_16, 32, 64, 128
public:
	cli_descriptor() {
		extern enum f_descriptor maxdescriptor();
		_flag = maxdescriptor(); // Set to os max file size
	};
	~cli_descriptor() {
		extern enum f_descriptor maxdescriptor();
		_flag = maxdescriptor();
	};
	void	flag(enum f_descriptor x) { _flag = x; };
	enum f_descriptor	flag() { return(_flag); };
	string print();
};

class cli_eid
{
private:
	string	_eid; // The eid
public:
	cli_eid() {
		_eid = "";

	};
	~cli_eid() {
		_eid = "";
	};
	void	eid(string x) { _eid = x; };
	string	eid() { return(_eid); };
	string print();
};

class cli_freespace
{
private:
	enum f_freespace	_flag; // Do we do bundles
public:
	cli_freespace() {
		_flag = F_FREESPACE_NO;
	};
	~cli_freespace() {
		_flag = F_FREESPACE_NO;
	};
	void	flag(enum f_freespace x) { _flag = x; };
	enum f_freespace	flag() { return(_flag); };
	string print();
};

class cli_get
{
private:
	string	_fname;	// The file to receive into
	sarnet::ip	_peer; // The peer to get from
public:
	cli_get() { };
	~cli_get() { this->zap(); };

	void	peer(sarnet::ip &p) { _peer = p; };
	sarnet::ip	*peer() { return &_peer; };
	void	fname(string s) { _fname = s; };
	string	fname() { return _fname; };
	void zap()
	{
		_fname = "";
		sarnet::ip tmp;
		_peer = tmp;
	}
};

class cli_ls
{
private:
	string	_dname;
	sarnet::ip	_peer;
public:
	cli_ls() { };
	~cli_ls() { this->zap(); };

	void	peer(sarnet::ip &p) { _peer = p; };
	sarnet::ip	*peer() { return &_peer; };
	void	dname(string s) { _dname = s; };
	string	dname() { return _dname; };
	void zap()
	{
		_dname = "";
		sarnet::ip tmp;
		_peer = tmp;
	}
};

class cli_peers
{
private:
	std::list<sarnet::ip *>	_peers;
public:
	cli_peers() { };
	~cli_peers() { this->zap(); };
		
	void zap() 
	{ 
		while (! _peers.empty()) 
		{
			delete _peers.front();
			_peers.pop_front(); 
		}
	};

	void	add(string s) { 
		sarnet::ip *tmp = new sarnet::ip(s); 
		(*this).remove(s); 
		_peers.push_back(tmp); 
	};

	void	remove(string s) 
	{
		sarnet::ip	tmp(s); 
		for (std::list<sarnet::ip *>::iterator i = _peers.begin(); i != _peers.end(); i++)
		{
			if (**i == tmp)
			{
				_peers.remove(*i);
				break;
			}
		}
	};
	std::list<sarnet::ip *>::iterator begin() { return(_peers.begin()); };
	std::list<sarnet::ip *>::iterator end() { return(_peers.end()); };

	string print();
};

class cli_put
{
private:
	string	_fname;
	sarnet::ip	_peer;
public:
	cli_put() { };
	~cli_put() { this->zap(); };

	void	peer(sarnet::ip &p) { _peer = p; };
	sarnet::ip	*peer() { return &_peer; };
	void	fname(string s) { _fname = s; };
	string	fname() { return _fname; };
	void zap()
	{
		_fname = "";
		sarnet::ip tmp;
		_peer = tmp;
	}
};

class cli_rm
{
private:
	string	_fname;
	sarnet::ip	_peer;
public:
	cli_rm() { };
	~cli_rm() { this->zap(); };

	void	peer(sarnet::ip &p) { _peer = p; };
	sarnet::ip	*peer() { return &_peer; };
	void	fname(string s) { _fname = s; };
	string	fname() { return _fname; };
	void zap()
	{
		_fname = "";
		sarnet::ip tmp;
		_peer = tmp;
	}
};

class cli_rmdir
{
private:
	string	_dname;
	sarnet::ip	_peer;
public:
	cli_rmdir() { };
	~cli_rmdir() { this->zap(); };

	void	peer(sarnet::ip &p) { _peer = p; };
	sarnet::ip	*peer() { return &_peer; };
	void	dname(string s) { _dname = s; };
	string	dname() { return _dname; };
	void zap()
	{
		_dname = "";
		sarnet::ip tmp;
		_peer = tmp;
	}
};

class cli_rx
{
private:
	enum f_rxwilling	_flag; // Do we do bundles
public:
	cli_rx() { _flag = F_RXWILLING_YES; };

	~cli_rx() { _flag = F_RXWILLING_YES; };
	void	flag(enum f_rxwilling x) { _flag = x; };
	enum f_rxwilling	flag() { return(_flag); };
	string print();
};

class cli_streams
{
private:
	enum f_streams	_flag; // Do we do bundles
public:
	cli_streams() { _flag = F_STREAMS_NO; };

	~cli_streams() { _flag = F_STREAMS_NO; };
	void	flag(enum f_streams x) { _flag = x; };
	enum f_streams	flag() { return(_flag); };
	string print();
};

class cli_timestamp
{
private:
	enum f_tstamp _flag; // Do we do timestamps
	enum t_timestamp	_ttype; // What type
public:
	cli_timestamp() {
		_flag = F_TIMESTAMP_NO;
		_ttype = T_TSTAMP_UDEF; };
	~cli_timestamp() {
		_flag = F_TIMESTAMP_NO;
		_ttype = T_TSTAMP_UDEF; };
	void flag(enum f_tstamp x) { _flag = x; };
	enum f_tstamp flag() { return(_flag); };
	void ttype(enum t_timestamp x) { _ttype = x; };
	enum t_timestamp ttype() { return(_ttype); };
	string print();
};

class cli_tx
{
private:
	enum f_txwilling 	_flag; // Do we do bundles
public:
	cli_tx() { _flag = F_TXWILLING_YES; };

	~cli_tx() { _flag = F_TXWILLING_YES; };
	void	flag(enum f_txwilling  x) { _flag = x; };
	enum f_txwilling 	flag() { return(_flag); };
	string print();
};

/*
 *****************************************************************************************
 */

// A command has a name, a usage and a function to call
class cmd 
{
typedef bool (cmd::*cmdptr)();
private:
	string	_cmd;
	string	_usage;
	string	_help;
	cmdptr	_function;
	vector<string>	_args;
public:
	// These are the command line handlers for each function
	bool cmd_help();
	bool cmd_beacon();
	bool cmd_bundles();
	bool cmd_checksum();
	bool cmd_debug();
	bool cmd_descriptor();
	bool cmd_exit();
	bool cmd_eid();
	bool cmd_freespace();
	bool cmd_get();
	bool cmd_ls();
	bool cmd_peers();
	bool cmd_put();
	bool cmd_rm();
	bool cmd_rmdir();
	bool cmd_rx();
	bool cmd_streams();
	bool cmd_timestamp();
	bool cmd_tx();

	cmd() {};
	cmd(string cmd, string usage, string help, cmdptr fp)
	{
		_cmd = cmd; // The base command
		_usage = usage; 
		_help = help;
		_function = fp; 
	};

	~cmd() { };

	cmd& operator =(const cmd &c)
	{
		_cmd = c._cmd;
		_usage = c._usage;
		_help = c._help;
		_function = c._function;
		return(*this);
	}

	bool operator ==(const cmd &rhs) { return(rhs._cmd == _cmd); };
	bool operator ==(string rhs) { return(rhs == _cmd); };
	bool operator !=(const cmd &rhs) { return !(rhs._cmd == _cmd); };
	bool operator !=(string rhs) { return !(rhs == _cmd); };

	string name() { return _cmd; };
	string usage() { return _usage; };
	string help() { return _help; };
	void setargs(vector<string>);
	void delargs();
	bool run() { return((this->*_function)()); };
	void print() { cout << _cmd << " " << _usage << endl; };
	void errprint() { cerr << _cmd << " " << _usage << endl; };
};

class cmds
{
private:

#define NCMDS (sizeof(_clist) / sizeof(cmd))

	const static int	_ncmds = 21; // NCMDS;

	cmd	_clist[_ncmds] = {
		{"?", 
			"", 
			"show valid commands. cmd ? show usage", 
			&cmd::cmd_help},
		{"beacon", 
			"beacon [off] [v4|v6|<ip>...] [secs]", 
			"send a beacon every secs", 
			&cmd::cmd_beacon},
	 	{"bundles", 
			"bundles [on|off]", 
			"we can handle bundles of files", 
			&cmd::cmd_bundles},
		{"checksum",
			"checksum [off|none|crc32|md5|sha1]",
			"set checksums required and type",
			&cmd::cmd_checksum},
		{"debug",
			"debug [off|0..9]",
			"set debug level 0..9",
			&cmd::cmd_debug},
		{"descriptor",
			"descriptor [off|16|32|64|128]",
			"advertise & set default descriptor size",
			&cmd::cmd_descriptor},
		{"eid",
			"eid [off] <eid>",
			"manually set the eid",
			&cmd::cmd_eid},
	 	{"exit", 
			"exit [0|1]", 
			"exit saratoga", 
			&cmd::cmd_exit},
	 	{"freespace", 
			"freespace [off|on]", 
			"do we advertise freespace", 
			&cmd::cmd_freespace},
		{"get",
			"get <peer> <filename>",
			"Get a file from a peer",
			&cmd::cmd_get},
		{"help", 
			"help", 
			"show commands", 
			&cmd::cmd_help},
		{"ls",
			"ls <peer> [<dirname>]",
			"Get a directory listing from a peer",
			&cmd::cmd_ls},
 		{"quit", 
			"quit [0|1]", 
			"exit saratoga", 
			&cmd::cmd_exit},
		{"peers",
			"peers [off] [<ip>...]",
			"List peers or Add/Remove peers",
			&cmd::cmd_peers},
		{"put",
			"put <peer> <filename>",
			"Send a file to a peer",
			&cmd::cmd_put},
		{"rm",
			"rm <peer> <filename>",
			"Remove a file from a peer",
			&cmd::cmd_rm},
		{"rmdir",
			"rmdir <peer> <dirname>",
			"Remove a directory from a peer",
			&cmd::cmd_rmdir},
		{"rx",
			"rx [on|off]", 
			"Saratoga can or cannot receive",
			&cmd::cmd_rx},
		{"streams",
			"streams [on|off]", 
			"Saratoga Can handle streams",
			&cmd::cmd_streams},
		{"timestamp",
			"timestamp [off|32|64|32_32|64_32|32_y2k]",
			"Timestamps off or what type to send",
			&cmd::cmd_timestamp},
		{"tx",
			"tx [on|off]", 
			"Saratoga will or wont transmit",
			&cmd::cmd_tx}
	};
public:
	cmds() { };

	~cmds() { };

	string usage(string); // Given a command string print its usage
	string cmdstr(string); // Given a command string print its string (will be same)
	bool runcmd(vector<string>);// Given command & arguments run the function to do it
					// the first vector[0] is the command to run
	string print(); // Print out the list of available commands
};

}; // Namespace saratoga

#endif // _CLI_H

/*
 * $Log: cli.h,v $
 * Revision 1.2  2015/01/09 22:09:49  charlesetsmith
 * Moved all Screen output to use curses. There are two windows, the top is status
 * and error messages. THe bottom is command line interaction. Also created
 * separate name spaces for curses (sarwin) and ip (sarip) so they can be
 * independant of the base saratoga code set. Fixed error in SConstruct
 * so that checksums now compiles properly and the checksums SConstruct
 * g++ options are same as the base code set. Huge number of changes but
 * the code set now operational. Remeber you have to "sudo ./saratoga"
 * as multicast requires you to be root to run.
 *
 * Revision 1.1.1.1  2014/10/15 11:55:35  lloydwood
 * Creating saratoga-vallona-dev.
 *
 * This is the Saratoga transfer protocol Vallona implementation,
 * under development by Charles Smith.
 *
 * Revision 1.3  2013/05/30 12:43:41  chas
 * LOts of changes to get timestamps going with having Y2K timestamps and
 * working on directory entry records
 *
 * Revision 1.2  2013/04/21 12:45:18  chas
 * MOre work on cli decided on arrays of strings. This will change to arrays of structs
 * when I get the command function pointers added.
 *
 * Revision 1.1  2013/04/17 03:02:18  chas
 * Beginning the CLI user interface. Have a vector holding the
 * valid cli options.
 *
 */

