/*
 * $Id: data.h,v 1.2 2015/01/09 22:09:49 charlesetsmith Exp $
 */

/*

 Copyright (c) 2011, Charles Smith
 All rights reserved.

 Redistribution and use in source and binary forms, with or without modification,
 are permitted provided that the following conditions are met:

    * Redistributions of source code must retain the above copyright notice, this
      list of conditions and the following disclaimer.
    * Redistributions in binary form must reproduce the above copyright notice, this
      list of conditions and the following disclaimer in the documentation and/or
      other materials provided with the distribution.
    * Neither the name of Vallona Networks nor the names of its contributors
      may be used to endorse or promote products derived from this software without
      specific prior written permission.

 THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND
 ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
 IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT,
 INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING,
 BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
 LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE
 OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED
 OF THE POSSIBILITY OF SUCH DAMAGE.

 */

#ifndef _DATA_H
#define _DATA_H

#include <iostream>
#include <cstring>
#include <string>
using namespace std;

#include "saratoga.h"
#include "sarflags.h"
#include "screen.h"
#include "timestamp.h"
#include "frame.h"

namespace saratoga
{
/*
 **********************************************************************
 * DATA
 **********************************************************************
 */

class data: public frame
{
private:
		flag_t		_flags;		// Data's Flags
        session_t	_session;	// THe session ID
        offset_t	_offset;	// How far into the transfer we are
        timestamp	*_timestamp;// If we have a timestamp what is it
        char		*_dbuf;		// THe buffer holding the data
        size_t		_dbuflen;		// How long the buffer is
public:

	// This is how we assemble a local data
	// To get ready for transmission
	// The optional timestamp is created within
	// this constructor. There is a data::timestamp()
	// to amend it if needed.
	data( enum f_descriptor,
		enum f_transfer,
		enum f_reqstatus,
		enum f_eod,
		enum f_tstamp, // If we want a timestamp
		session_t,	// Session NUmber
		offset_t,	// offset 
		char *, 	// payload
		size_t);	// length of payload

	// We have received a remote frame that is DATA
	data(char *,	// Pointer to buffer received
		size_t);	// Total length of buffer

	~data() {
		delete[] _payload;
		_paylen = 0;
		if (_timestamp != NULL)
			delete _timestamp;
		_session = 0;
		_offset = 0;
		_dbuf = NULL;
		_dbuflen = 0;
		_flags = 0;
	};

	// Get various flags applicable to data
	enum f_transfer		transfer();
	enum f_reqstatus		reqstatus();
	enum f_eod		eod();
	enum f_tstamp		tstamp();

	// Set various flags applicable to data
	enum f_transfer		transfer(f_transfer);
	enum f_reqstatus		reqstatus(f_reqstatus);
	enum f_eod		eod(f_eod);
	enum f_tstamp		tstamp(f_tstamp);
	
	// What is the session number for this transaction
	session_t	session() { return _session; };

	// How far into the transfer are we and we can set it
	offset_t	offset() { return _offset; };
	offset_t	offset(offset_t o) { _offset = o; return _offset; };

	// The data frames flags
	flag_t	flags() { return _flags; };

	// Yes I am a data frame
	f_frametype type() { return F_FRAMETYPE_DATA; };

	// The data frames payload
	char 	*payload() { return _payload; };
	size_t	paylen() { return _paylen; };

	// What is the length of the data buffer in this frame
	char	*dbuf() { return _dbuf; };
	size_t	dbuflen() { return _dbuflen; };

	ssize_t rx() {
		string s = "DATA RX: ";
		s += this->print();
		cout << s << endl;
		return(-1);
	};

	ssize_t tx(sarnet::udp *sock) {
		string s = "DATA TX: ";
		s+= this->print();
		cout << s << endl;
		return(-1);
	};

	string	printflags();
	string	print();
};

} // Namespace saratoga

#endif // _DATA_H

/*
 * $Log: data.h,v $
 * Revision 1.2  2015/01/09 22:09:49  charlesetsmith
 * Moved all Screen output to use curses. There are two windows, the top is status
 * and error messages. THe bottom is command line interaction. Also created
 * separate name spaces for curses (sarwin) and ip (sarip) so they can be
 * independant of the base saratoga code set. Fixed error in SConstruct
 * so that checksums now compiles properly and the checksums SConstruct
 * g++ options are same as the base code set. Huge number of changes but
 * the code set now operational. Remeber you have to "sudo ./saratoga"
 * as multicast requires you to be root to run.
 *
 * Revision 1.1.1.1  2014/10/15 11:55:35  lloydwood
 * Creating saratoga-vallona-dev.
 *
 * This is the Saratoga transfer protocol Vallona implementation,
 * under development by Charles Smith.
 *
 * Revision 1.11  2014/10/04 03:04:36  chas
 * Just too much to explain all. We are moving to using ncurses for terminal I/O
 * We have the basis for the main saratoga loop going
 * Beacon NEARLY works
 * Lots of command line additions (hence the need to get ncurses going. THere
 * is just WAY TOO MUCH screen stuff/debug going on. So Command line is
 * in the bottom window and Error/Status Outputs are in the top window.
 *
 * Revision 1.10  2013/12/02 07:13:48  chas
 * *** empty log message ***
 *
 * Revision 1.9  2013/10/04 02:04:18  chas
 * Finally fixed the damn issues with METADATA. Problem was that I was not
 * updating the Frame copy of the metadata. i.e. I needed to copy
 * to _payload and _paylen in the creator for metadata.
 *
 * Revision 1.8  2013/06/05 01:44:22  chas
 * Sorted out frame's
 *
 * Revision 1.7  2013/06/01 07:32:40  chas
 * Some issues with frame virtualisation. Perhaps this was a bad idea pulling the flags into the frame class... Ill think it over with a couple of scotches (single malt of course).
 *
 * Revision 1.6  2013/06/01 06:30:09  chas
 * Removed _flags from each individual frame type (BEACON, DATA, METADATA...) and ut the _flags in a friend frame class.
 *
 * Revision 1.5  2013/03/28 08:22:36  chas
 * Found an error in flags. Some wrong bit shifting for data flags
 * Progressing along with data class. And making printf's a bit more
 * portable with PRIuXX's for _session and _offset types
 *
 * Revision 1.4  2013/03/27 11:09:33  chas
 * Working on Data polymorph frame type now. Setting and getting flags
 * applicable to data and the optional timestamp. Some more thinking
 * on how I stamp the timestamp. THink it is best done within the
 * class constructor with an optional data.timestamp() to update it to
 * now when needed.
 *
 * Revision 1.3  2013/03/24 22:00:26  chas
 * Added data.h and split out beacon.h from frame.h
 *
 */

