/*
 * $Id: fileio.h,v 1.2 2015/01/09 22:09:49 charlesetsmith Exp $
 */

/*
 
 Copyright (c) 2011, Charles Smith
 All rights reserved.

 Redistribution and use in source and binary forms, with or without modification, 
 are permitted provided that the following conditions are met:

    * Redistributions of source code must retain the above copyright notice, this 
      list of conditions and the following disclaimer.
    * Redistributions in binary form must reproduce the above copyright notice, this
      list of conditions and the following disclaimer in the documentation and/or 
      other materials provided with the distribution.
    * Neither the name of Vallona Networks nor the names of its contributors 
      may be used to endorse or promote products derived from this software without 
      specific prior written permission.

 THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND 
 ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED 
 WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
 IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, 
 INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, 
 BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, 
 DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF 
 LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE 
 OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED 
 OF THE POSSIBILITY OF SUCH DAMAGE.

 */

#ifndef _FILEIO_H
#define _FILEIO_H

#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>
#include <limits>
#include <unistd.h>
#include <errno.h>
// #include <stdio.h>
#include <string>

using namespace std;

#include "screen.h"
#include "saratoga.h"

/*
 * Handle debug and error output.
 * Flexible as you can set the level and also what file you want to talk to
 * defaults are stderr of course. When setting a new fp make sure we dont close stdout and stderr
 */
namespace saratoga
{

enum rorw
{
	FILE_UNDEF = -1,
	FILE_READ = 0,
	FILE_WRITE = 1
};

class fileio
{
private:
	string		_fname; // THe file name
	enum rorw	_rorw; // Open for reading or writing
	int		_fd;	 // Local file descriptor
public:
	fileio() { _fd = -1; _rorw = FILE_UNDEF; _fname = ""; };
	fileio(string fname, enum rorw rw);

	int fcreat(string fname);
	int fopen(string fname);
	void fclose();
	int funlink();

	offset_t fseek(offset_t offset);
	offset_t flen();
	ssize_t	fread(void *b, size_t cnt);
	ssize_t fwrite(const void *b, size_t cnt);

	~fileio() { if (_fd > 2) close(_fd); _fd = -1; _rorw = FILE_UNDEF; };
};

} // namespace saratoga

#endif // _FILEIO_H

/*
 * $Log: fileio.h,v $
 * Revision 1.2  2015/01/09 22:09:49  charlesetsmith
 * Moved all Screen output to use curses. There are two windows, the top is status
 * and error messages. THe bottom is command line interaction. Also created
 * separate name spaces for curses (sarwin) and ip (sarip) so they can be
 * independant of the base saratoga code set. Fixed error in SConstruct
 * so that checksums now compiles properly and the checksums SConstruct
 * g++ options are same as the base code set. Huge number of changes but
 * the code set now operational. Remeber you have to "sudo ./saratoga"
 * as multicast requires you to be root to run.
 *
 * Revision 1.1.1.1  2014/10/15 11:55:35  lloydwood
 * Creating saratoga-vallona-dev.
 *
 * This is the Saratoga transfer protocol Vallona implementation,
 * under development by Charles Smith.
 *
 * Revision 1.2  2013/04/01 09:57:55  chas
 * Working on dirent. More work to do yet in working out a default
 * descriptor size based upon the size of the incoming file. This
 * may be overridden once we have a session defined. stat call
 * is nearly done.
 *
 * Revision 1.1.1.1  2013/03/09 07:35:04  chas
 * Saratoga C++ Primary Implementation
 * Charles Smith charlesetsmith@me.com
 *
 * Revision 1.1  2011-01-18 10:15:43  chas
 * Refining logging and error printing classes
 *
 */

