/*
 * $Id: frame.h,v 1.2 2015/01/09 22:09:49 charlesetsmith Exp $
 */

/*
 
 Copyright (c) 2011, Charles Smith
 All rights reserved.

 Redistribution and use in source and binary forms, with or without modification, 
 are permitted provided that the following conditions are met:

    * Redistributions of source code must retain the above copyright notice, this 
      list of conditions and the following disclaimer.
    * Redistributions in binary form must reproduce the above copyright notice, this
      list of conditions and the following disclaimer in the documentation and/or 
      other materials provided with the distribution.
    * Neither the name of Vallona Networks nor the names of its contributors 
      may be used to endorse or promote products derived from this software without 
      specific prior written permission.

 THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND 
 ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED 
 WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
 IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, 
 INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, 
 BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, 
 DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF 
 LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE 
 OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED 
 OF THE POSSIBILITY OF SUCH DAMAGE.

 */

#ifndef _FRAME_H
#define _FRAME_H

#include <iostream>
#include <cstring>
#include <string>
using namespace std;

#include "saratoga.h"
#include "sarflags.h"
#include "screen.h"
#include "timestamp.h"
#include "ip.h"

namespace saratoga 
{

/*
 * A received or sent saratoga frame has flags and a payload
 * This is the master class for all of the frame types
 * BEACON, DATA, STATUS, REQUEST, METADATA
 */
class frame
{
protected:
	char		*_payload;	// A frames payload
	size_t		_paylen;	// This is how long the payload is
	bool		_badframe;	// Set if this is a bad saratoga frame
public:

	virtual ~frame() { };		// Its all deleted in the sub-class

	/* What is the length of the payload in this frame */
	size_t	paylen() const {
		cout << "Frame Paylen == " << _paylen << endl;
		return _paylen;
	};

	/* Send me a pointer to the payload */
	char	*payload() const {
		return(_payload);
	};

	/* What type of frame is this */
	virtual f_frametype type() { return this->type(); };

	// These functions are virtual and ALL code must be here
	// in the class declaration

	/* This frames saratoga flags */
	virtual flag_t	flags() { return this->flags(); };

	/* Send a frame */
	virtual ssize_t tx(sarnet::udp *sock) = 0;

	/* Receive a frame */
	virtual ssize_t rx() = 0;

	/* Print the details of the frame */
	virtual string	print() { return this->print(); };
};

} // Namesapce saratoga

#endif // _FRAME_H

/*
 * $Log: frame.h,v $
 * Revision 1.2  2015/01/09 22:09:49  charlesetsmith
 * Moved all Screen output to use curses. There are two windows, the top is status
 * and error messages. THe bottom is command line interaction. Also created
 * separate name spaces for curses (sarwin) and ip (sarip) so they can be
 * independant of the base saratoga code set. Fixed error in SConstruct
 * so that checksums now compiles properly and the checksums SConstruct
 * g++ options are same as the base code set. Huge number of changes but
 * the code set now operational. Remeber you have to "sudo ./saratoga"
 * as multicast requires you to be root to run.
 *
 * Revision 1.1.1.1  2014/10/15 11:55:35  lloydwood
 * Creating saratoga-vallona-dev.
 *
 * This is the Saratoga transfer protocol Vallona implementation,
 * under development by Charles Smith.
 *
 * Revision 1.10  2014/10/04 03:04:36  chas
 * Just too much to explain all. We are moving to using ncurses for terminal I/O
 * We have the basis for the main saratoga loop going
 * Beacon NEARLY works
 * Lots of command line additions (hence the need to get ncurses going. THere
 * is just WAY TOO MUCH screen stuff/debug going on. So Command line is
 * in the bottom window and Error/Status Outputs are in the top window.
 *
 * Revision 1.9  2013/06/05 01:44:21  chas
 * Sorted out frame's
 *
 * Revision 1.8  2013/06/01 07:32:40  chas
 * Some issues with frame virtualisation. Perhaps this was a bad idea pulling the flags into the frame class... Ill think it over with a couple of scotches (single malt of course).
 *
 * Revision 1.7  2013/06/01 06:38:23  chas
 * Added _flags to frame.h so the _payload is no the rest of the data and its length
 *
 * Revision 1.6  2013/06/01 00:16:55  chas
 * Fixed bug with frame conversion
 *
 * Revision 1.5  2013/03/24 22:00:26  chas
 * Added data.h and split out beacon.h from frame.h
 *
 * Revision 1.4  2013/03/23 12:01:00  chas
 * Finally sorted out polymorhic "frame" class and beacon all done
 * so now I can move onto similar for other frame types
 *
 * Revision 1.3  2013/03/15 12:17:34  chas
 * Moving along with the polymorphism. Beacon nearly done
 *
 * Revision 1.2  2013/03/15 04:53:41  chas
 * Moving frames to polymophism so we have a single base class of frame and
 * then polymorph frame types of beacon and data. As we move forward
 * I will add the other saratoga frame types. Lots more work to do here yet
 * as I havn't got the base class sorted yet.
 *
 * Revision 1.1.1.1  2013/03/09 07:35:04  chas
 * Saratoga C++ Primary Implementation
 * Charles Smith charlesetsmith@me.com
 *
 */
