/*
 * $Id: ip.cpp,v 1.2 2015/01/09 22:09:49 charlesetsmith Exp $
 */

/*

 Copyright (c) 2014, Charles Smith
 All rights reserved.

 Redistribution and use in source and binary forms, with or without modification,
 are permitted provided that the following conditions are met:

    * Redistributions of source code must retain the above copyright notice, this
      list of conditions and the following disclaimer.
    * Redistributions in binary form must reproduce the above copyright notice, this
      list of conditions and the following disclaimer in the documentation and/or
      other materials provided with the distribution.
    * Neither the name of Vallona Networks nor the names of its contributors
      may be used to endorse or promote products derived from this software without
      specific prior written permission.

 THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND
 ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
 IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT,
 INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING,
 BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
 LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE
 OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED
 OF THE POSSIBILITY OF SUCH DAMAGE.

 */
#include <fcntl.h>
#include <iostream>
#include <cstring>
#include <string>
#include "screen.h"
#include "ip.h"

using namespace std;

// #include "timestamp.h"
/*
 * Everything to do with ip and udp networking basics go here under sarnet namespace
 */
namespace sarnet
{

// Given a string create IP
ip::ip(string addr)
{
	sarwin::screen	scr;

	// char *addrs = (char *) malloc(addr.length() + 1);
	char *addrs = new char[addr.length() + 1];
	strcpy(addrs, addr.c_str());
	if (addr.find(":") != std::string::npos)
		_family = AF_INET6;
	else if (addr.find(".") != std::string::npos)
		_family = AF_INET;
	else
	{
		scr.error("ip(%s):Invalid IP Address\n", addrs); 
		_family = OTHER;
		bzero(&_ip, sizeof(union in_storage));
		delete addrs;
		return;
	}
	/* Convert existing string to in_addr */
	if (inet_pton(_family, addrs, &_ip) != 1)
	{
		scr.error("ip(%s):Invalid IP Address\n", addrs); 
		_family = OTHER;
		bzero(&_ip, sizeof(union in_storage));
	}
	delete addrs;
	return;
}

string
ip::print()
{
	// static string ret;
	string ret;
	char	*s;

	if (this == NULL)
		return("");
	if (_family == OTHER)
		return("OTHER");
	if (_family == AF_INET)
	{
		s = new char [INET_ADDRSTRLEN];
		inet_ntop(_family, &_ip.v4, s, INET_ADDRSTRLEN);
		ret = s;
		delete s;
		return(ret);
	}
	if (_family == AF_INET6)
	{
		s = new char [INET6_ADDRSTRLEN];
		inet_ntop(_family, &_ip.v6, s, INET6_ADDRSTRLEN);
		ret = s;
		delete s;
		return(ret);
	}
	return("INVALID");
}

// List of local IP Addresses for this machine
netifaces::netifaces()
{
	string	s;
	sarwin::screen	scr;

	// struct addrinfo	hint, *res, *p;
	struct ifaddrs *res, *p;
	int	status;
	char	ipstr[INET6_ADDRSTRLEN];

	if ((status = getifaddrs(&res)) != 0)	
	{
		scr.perror(errno, "Cannot get local machine IP Addresses");
		exit(1);
	}

	// Loop through my ip addresses looking for V4 & V6 ones only
	for (p = res; p != NULL; p = p->ifa_next)
	{
		void *addr;
		string tmpifname = p->ifa_name;
		if (p->ifa_addr->sa_family == AF_INET)
		{
			struct sockaddr_in *ipv4 = (struct sockaddr_in *) p->ifa_addr;
			addr = &(ipv4->sin_addr);
		}
		else if (p->ifa_addr->sa_family == AF_INET6)
		{
			struct sockaddr_in6 *ipv6 = (struct sockaddr_in6 *) p->ifa_addr;
			addr = &(ipv6->sin6_addr);
		}
		else
			continue;
		// Find the index interface associated with this ip address
		inet_ntop(p->ifa_addr->sa_family, addr, ipstr, INET6_ADDRSTRLEN);
		string tmpipstr = ipstr;
		netiface *tmpiface = new netiface(tmpipstr, tmpifname);
		_iface.push_back(tmpiface);
	}
	s = "LOCAL INTERFACES ARE: " + this->print();
	scr.debug(1, s);
	freeifaddrs(res);
}

// Return string with : delimeted MAC address
string
netiface::mac()
{
	string s; 
	char str[128];
	string sip = _ifip->print();
	sprintf(str, "%02x:%02x:%02x:%02x:%02x:%02x", 
		(unsigned char) _mac[0], 
		(unsigned char) _mac[1], 
		(unsigned char) _mac[2], 
		(unsigned char) _mac[3], 
		(unsigned char) _mac[4], 
		(unsigned char) _mac[5]);
	s += str;
	return(s);
}
	
string
netiface::print() 
{ 
	string s; 
	char str[128];
	string sip = _ifip->print();
	sprintf(str, "%s:%d=%s", 
		_ifname.c_str(), 
		_ifindex, 
		sip.c_str());
	s += str;
	s += " MAC=";
	s += this->mac();
	return(s);
}

// Given an interface name fill in and return *to the MAC address
char *
if_nametomac(char *mac, string ifname)
{
	sarwin::screen	scr;

	struct ifreq	ifx;
	int	fd = -1;
	char	s[IF_NAMESIZE];
	strcpy(s, ifname.c_str());

	for (int i = 0; i < 6; i++)
		mac[i] = ifx.ifr_addr.sa_data[i];
	if ((fd = socket(PF_INET, SOCK_DGRAM, IPPROTO_IP)) < 0)
	{
		scr.error("Can't create socket to do ioctl for MAC lookup\n");
		return(mac);
	}
	strcpy(ifx.ifr_name, s);
	if (ioctl(fd, SIOCGIFHWADDR, &ifx) == 0)
	{
		for (int i = 0; i < 6; i++)
			mac[i] = ifx.ifr_addr.sa_data[i];
	}
	else
		scr.error("Can't ioctl to get MAC address\n");
	close(fd);
	return(mac);
}

/*
 ************************************************************************
 * UDP
 ************************************************************************
 */

// Create UDP Socket port for dest ip, fill in the ip structure & set fd
// Used when writing to a particular socket at a destination
udp::udp(string addr, uint16_t port)
{
	const int	on = 1;
	sarwin::screen	scr;
	char		addrstr[128];
	struct sockaddr_in *in = (sockaddr_in *) &_sa;
	struct sockaddr_in6 *in6 = (sockaddr_in6 *) &_sa;

	struct protoent	*proto;

	ip	ipa(addr);
	proto = getprotobyname("UDP");
	strcpy(addrstr, addr.c_str());

	switch(ipa.family())
	{
	case AF_INET:
		_fd = socket(AF_INET, SOCK_DGRAM, proto->p_proto);
		if (_fd == -1)
		{
			scr.perror(errno, "Can't create socket");
			return;
		}
		memcpy(&in->sin_addr, ipa.addr4(), sizeof(struct in_addr));
		in->sin_family = AF_INET;
		in->sin_port = htons(port);
		break;
	case AF_INET6:
		_fd = socket(AF_INET6, SOCK_DGRAM, proto->p_proto);
		if (_fd == -1)
		{
			scr.perror(errno, "Can't create socket");
			return;
		}
		memcpy(&in6->sin6_addr, ipa.addr6(), sizeof(struct in6_addr));
		in6->sin6_family = AF_INET6;
		in6->sin6_port = htons(port);
		in6->sin6_flowinfo = htonl(0);
		break;
	default:
		_fd = -1;
		scr.error("Can't create socket: Invalid family\n");
		return;
	}
	if (setsockopt(_fd, SOL_SOCKET, SO_REUSEADDR, &on, sizeof(on)) == -1)
	{
		scr.error("ip:udp(%s fd=%d port=%" PRIu16 "): Can't setsockopt to SO_REUSEADDR\n",
				addrstr, _fd, port);
		close(_fd);
		_fd = -1;
		return;
	}
	_readytotx = false; 
}

udp::udp(struct sockaddr_storage *p)
{
	sarwin::screen	scr;
	
	_fd = -1; // No fd for this type of udp
	_buf.empty(); // No buffers either
	bzero(&_sa, sizeof (struct sockaddr_storage)); 
	switch (p->ss_family)
	{
	case AF_INET:
		memcpy(&_sa, p, sizeof (struct sockaddr_in));
		break;
	case AF_INET6:
		memcpy(&_sa, p, sizeof (struct sockaddr_in6));
		break;
	default:
		scr.error("Invalid Frame Type for udp");
		return;
	}
	_readytotx = false;
}

// Create multicast UDP Socket port for a local ip (*p)
// That has already been open and bound
// This is for multicasts
udp::udp(enum mcast_dir dir, udp *p, string addr, uint16_t port)
{
	sarwin::screen	scr;
	char		addrstr[128];
	struct ip_mreq	group; // The IPv4 multicast group
	struct ipv6_mreq	group6; // The IPv6 multicast group
	int on = 1;	// Allow socket to be reused

	// The multicast address
	struct sockaddr_in *in = (sockaddr_in *) &_sa;
	struct sockaddr_in6 *in6 = (sockaddr_in6 *) &_sa;

	// The local address to bind to	
	struct sockaddr_in *local = (sockaddr_in *) &p->_sa;
	struct sockaddr_in6 *local6 = (sockaddr_in6 *) &p->_sa;

	struct protoent	*proto;

	// Get the interface index information	
	netifaces interfaces;
	netiface *firstv6 = interfaces.first(AF_INET6);
	// netiface *firstv4 = interfaces.first(AF_INET);

	proto = getprotobyname("UDP");
	ip	ipa(addr);
	strcpy(addrstr, addr.c_str());

	switch(ipa.family())
	{
	case AF_INET:
		_fd = socket(AF_INET, SOCK_DGRAM, proto->p_proto);
		if (_fd == -1)
		{
			scr.perror(errno, "AF_INET Can't create socket");
			return;
		}
		memcpy(&in->sin_addr, ipa.addr4(), sizeof(struct in_addr));
		in->sin_family = AF_INET;
		in->sin_port = htons(port);
		if (dir == MCAST_OUT)
		{
			// Bind it to local v4 fd
			if (setsockopt(_fd, SOL_SOCKET, IP_MULTICAST_IF,
				(void *) &local->sin_addr, sizeof (struct in_addr)) < 0)
			{
				string emsg = "Trying to bind multicast " + ipa.print() + " to " + p->straddr();
				scr.error(emsg);
				scr.perror(errno, "MCAST_OUT v4 ip:udp(%s fd=%d, port=%" PRIu16 ") Can't setsockopt to IP_MULTICAST_IF",
					addrstr, _fd, port);
				close(_fd);
				_fd = -1;
				return;
			}
		}
		else // MCAST_IN
		{
			if (setsockopt(_fd, SOL_SOCKET, SO_REUSEADDR, (void *) &on, sizeof(on)) < 0)
			{
				scr.perror(errno, "MCAST_IN v4 ip:udp(%s fd=%d, port=%" PRIu16 ") Can't set SO_REUSEADDR",
					addrstr, _fd, port);
				close(_fd);
				_fd = -1;
				return;
			}
			if (bind(_fd, (struct sockaddr *) local, sizeof (struct sockaddr_in)) != 0)
			{
				scr.perror(errno, "MCAST_IN v4 ip:udp(%s fd=%d, port=%" PRIu16 ") Can't bind to local",
					addrstr, _fd, port);
				close(_fd);
				_fd = -1;
				return;
			}
			memcpy(&group.imr_multiaddr, ipa.addr4(), sizeof (struct in_addr));
			memcpy(&group.imr_interface, &local->sin_addr, sizeof(struct in_addr));
			if (setsockopt(_fd, SOL_SOCKET, IP_ADD_MEMBERSHIP,
				(void *) &group, sizeof(group)) < 0)
			{
				scr.perror(errno, "MCAST_IN v4 ip:udp(%s fd=%d, port=%" PRIu16 ") Can't add to multicast group",
					addrstr, _fd, port);
				close(_fd);
				_fd = -1;
				return;
			}
		}
		break;
	case AF_INET6:
		_fd = socket(AF_INET6, SOCK_DGRAM, proto->p_proto);
		if (_fd == -1)
		{
			scr.perror(errno, "AF_INET6 Can't create socket");
			return;
		}
		memcpy(&in6->sin6_addr, ipa.addr6(), sizeof(struct in6_addr));
		in6->sin6_family = AF_INET6;
		in6->sin6_port = htons(port);
		in6->sin6_flowinfo = htonl(0);
		in6->sin6_scope_id = 0; // htonl(firstv6->ifindex());
		if (dir == MCAST_OUT)
		{
			// Bind it to local v6 fd
			if (setsockopt(_fd, SOL_SOCKET, IP_MULTICAST_IF,
				(void *) &local6->sin6_addr, sizeof (struct in6_addr)) < 0)
			{
				string emsg = "Trying to bind multicast " + ipa.print() + " to " + p->straddr();
				scr.error(emsg);
				scr.perror(errno, "MCAST_OUT v6 ip:udp(%s fd=%d, port=%" PRIu16 "): Can't setsockopt to IP_MULTICAST_IF",
					addrstr, _fd, port);
				close(_fd);
				_fd = -1;
				return;
			}
		}
		else // MCAST_IN
		{
			if (setsockopt(_fd, SOL_SOCKET, SO_REUSEADDR, (void *) &on, sizeof(on)) < 0)
			{
				scr.perror(errno, "MCAST_IN v6 ip:udp(%s fd=%d, port=%" PRIu16 ") Can't set SO_REUSEADDR",
					addrstr, _fd, port);
				close(_fd);
				_fd = -1;
				return;
			}
			if (bind(_fd, (struct sockaddr *) local6, sizeof (struct sockaddr_in6)) != 0)
			{
				string emsg = "Trying to bind to local v6 address " + p->print();
				scr.error(emsg);
				scr.perror(errno, "MCAST_IN v6 ip:udp(%s fd=%d, port=%" PRIu16 ") Can't bind to local6",
					addrstr, _fd, port);
				close(_fd);
				_fd = -1;
				return;
			}
			memcpy(&group6.ipv6mr_multiaddr, ipa.addr6(), sizeof (struct in6_addr));
			scr.msg("Setting V6 Index to %d", firstv6->ifindex());
			group6.ipv6mr_interface = 0; // firstv6->ifindex();
			if (setsockopt(_fd, SOL_SOCKET, IP_ADD_MEMBERSHIP,
				(void *) &group6, sizeof(group6)) < 0)
			{
				scr.perror(errno, "ip:udp(%s fd=%d, port=%" PRIu16 ") Can't add to multicast group",
					addrstr, _fd, port);
				close(_fd);
				_fd = -1;
				return;
			}
		}
		break;
	default:
		scr.error("ip:udp(%s fd=%d, port=%" PRIu16 ") Invalid family",
			addrstr, _fd, port);
		_fd = -1;
		break;
	}
	string errmsg = p->print() + " bound to multicast " + addr;
	scr.msg(errmsg);
	_readytotx = false;
	return;
}

// Open and bind the _fd to incoming UDP frames on a port (socket)
// so we can send and receive
udp::udp(int protocol, uint16_t port)
{
	sarwin::screen	scr;
	const int	on = 1;
	struct protoent	*proto;
	struct sockaddr_in *in = (sockaddr_in *) &_sa;
	struct sockaddr_in6 *in6 = (sockaddr_in6 *) &_sa;

	proto = getprotobyname("UDP");

	bzero(&_sa, sizeof(struct sockaddr_storage));
	switch(protocol)
	{
	case AF_INET:
		_fd = socket(AF_INET, SOCK_DGRAM, proto->p_proto);
		in->sin_family = AF_INET;
		in->sin_addr.s_addr = htonl(INADDR_ANY);
		in->sin_port = htons(port);
		break;
	case AF_INET6:
		_fd = socket(AF_INET6, SOCK_DGRAM, proto->p_proto);
		in6->sin6_family = AF_INET6;
		in6->sin6_addr = in6addr_any;
		in6->sin6_port = htons(port);
		break;
	default:
		scr.error("ip::udp(%d, %" PRIu16 "): Bad protocol only AF_INET or AF_INET6\n", 
			protocol, port);
		_fd = -1;
		return;
	}
	if (_fd == -1)
	{
		scr.error("ip:udp(%" PRIu16 "): Can't create socket\n", 
			port);
		return;
	}
	if (setsockopt(_fd, SOL_SOCKET, SO_REUSEADDR, &on, sizeof(on)) == -1)
	{
		scr.error("ip:udp(%d, %" PRIu16 "): Can't setsockopt to SO_REUSEADDR\n",
				_fd, port);
		if (_fd != -1)
			close(_fd);
		_fd = -1;
		return;
	}

	// Now bind the socket to the fd	
	switch(protocol)
	{
	case AF_INET:
		if (bind(_fd, (struct sockaddr *) &_sa, sizeof(struct sockaddr_in)) != 0)
		{
			scr.error("udp::bind(%d, %" PRIu16 "): AF_INET Can't bind socket",
				_fd, port);
			close(_fd);
			_fd = -1;
		}
		break;
	case AF_INET6:
		if (setsockopt(_fd, IPPROTO_IPV6, IPV6_V6ONLY, &on, sizeof(on)) == -1)
			scr.error("ip:udp(%d, %" PRIu16 "): Can't setsockopt to IPV6_V6ONLY\n",
				_fd, port);
		if (bind(_fd, (struct sockaddr *) &_sa, sizeof(struct sockaddr_in6)) != 0)
		{
			scr.perror(errno, "udp::bind(%d, %" PRIu16 "): AF_INET6 Can't bind socket",
				_fd, port);
			close(_fd);
			_fd = -1;
		}
		break;
	default:
		break;
	}
}

// Close down the Socket
udp::~udp()
{
	if (_fd >= 0)
	{
		shutdown(_fd, SHUT_RDWR);
		close(_fd);
		_fd = -1;
		_readytotx = false;
	}
	this->zap();
}

// Should we do a htons here CHECK IT!
uint16_t
udp::port()
{
	struct sockaddr_in *in = (sockaddr_in *) &_sa;
	struct sockaddr_in6 *in6 = (sockaddr_in6 *) &_sa;

	switch(this->family())
	{
	case AF_INET:
		return(ntohs(in->sin_port));
	case AF_INET6:
		return(ntohs(in6->sin6_port));
	default:
		return(0);
	}
}

int
udp::tx()
{
	sarwin::screen		scr;
	// static socklen_t	tolen;
	socklen_t	tolen;
	ssize_t			nwritten;
	int			flags = MSG_DONTWAIT;
	int	bcount = 0;	// # bytes written

	switch(this->family())
	{
	case AF_INET6:
		tolen = sizeof(struct sockaddr_in6);
		break;
	case AF_INET:
		tolen = sizeof(struct sockaddr_in);
		break;
	default:
		scr.error("udp::tx(): Bad protocol AF_INET & AF_INET6 supported\n");
		return(-1);
	}

	// We only send one buffer at a time then return
	// that way all sockets writing get an even share
	// of the link on the main select() loop
	if (! _buf.empty())
	{
		_readytotx = true;
		udpbuffer tmp(_buf.front());
		char	*b = tmp.buf();
		ssize_t	blen = tmp.len();
		if (blen != 0)
		{
			nwritten = sendto(_fd, b, blen, flags, (struct sockaddr *) &_sa, tolen);
			if (nwritten < 0)
			{
				int err = errno;
				string adr = this->straddr();
				scr.perror(err, "udp::tx(): Cannot write to %s Port %d\n",
					adr.c_str(), this->port());
			}
			else
			{
				string adr = this->straddr();
				scr.debug(0, "udp::tx(): Wrote %d bytes to %s Port %d",
					nwritten,
					adr.c_str(), this->port());
				bcount += nwritten;
			}
		}
		_buf.pop_front();
	}
	else
		_readytotx = false;
	// Check for last pop our buffer is empty
	if (_buf.empty())
		_readytotx = false;

	// Send back total bytes written
	return(bcount);
}

ssize_t
udp::rx(char *b)
{
	sarwin::screen		scr;
	string		s;
	// static socklen_t	len = sizeof(struct sockaddr_storage);
	socklen_t	len = sizeof(struct sockaddr_storage);
	ssize_t			nread;
	int			flags = MSG_DONTWAIT;
	struct sockaddr_storage	addrbuf;

	s = "Receiving interface " + this->print();
	scr.debug(2, s);
	switch (this->family())
	{
	case AF_INET:
		len = sizeof (struct sockaddr_in);
		break;
	case AF_INET6:
		len = sizeof (struct sockaddr_in6);
		break;
	default:
		scr.error("udp::rx() Invalid family");
		return -1;
	}

//	struct sockaddr *from = (struct sockaddr *) &_sa;
	nread = recvfrom(_fd, b, 9000, flags, (struct sockaddr *) &addrbuf, &len);

	if (len == sizeof(sockaddr_in))
	{
		udp	fromv4(&addrbuf);
		s = "Received v4 from " + fromv4.print();
		scr.debug(1, s);
	}
	else if (len == sizeof(sockaddr_in6))
	{
		udp	fromv6(&addrbuf);
		s = "Received v6 from " + fromv6.print();
		scr.debug(1, s);
	}
	else
	{
		scr.error("Invalid read len size: %d\n", len);
	}
	if (nread < 0)
	{
		int err = errno;
		scr.perror(err, "udp::rx(): Cannot read\n");
	}
	return(nread);
}

// Return string of the IP address
string
udp::straddr()
{
	sarwin::screen	scr;
	string ret;
	struct sockaddr_in *in = (struct sockaddr_in *) &_sa;
	struct sockaddr_in6 *in6 = (struct sockaddr_in6 *) &_sa;
	char *s;
	
	switch(this->family())
	{
	case AF_INET:
		s = new char [INET_ADDRSTRLEN];
		// s = (char *) malloc(INET_ADDRSTRLEN);
		inet_ntop(this->family(), &in->sin_addr, s, INET_ADDRSTRLEN);
		ret = s;
		free(s);
		return(ret);
	case AF_INET6:
		s = new char [INET6_ADDRSTRLEN];
		// s = (char *) malloc(INET6_ADDRSTRLEN);
		inet_ntop(this->family(), &in6->sin6_addr, s, INET6_ADDRSTRLEN);
		ret = s;
		free(s);
		return(ret);
	default:
		scr.error("udp::straddr() Invalid IP family\n");
		return("OTHER");
	}
}

bool
udp::setopt(int option, void *optval, socklen_t optsize) {
	sarwin::screen	msg;

	if (setsockopt(_fd, SOL_SOCKET, option, optval, optsize) < 0)
	{
		msg.perror(errno, "Can't setsockopt %d for socket %d\n", 
			option, _fd);
		return(0);
	}
	return(1);
};

// Is the option set OK ?
bool 
udp::getopt(int option, void *optval, socklen_t *optlen) {
	sarwin::screen	msg;
	
	if (getsockopt(_fd, SOL_SOCKET, option, optval, optlen) < 0)
	{
		msg.perror(errno, "Can't getsockopt %d for socket %d\n", 
			option, _fd);
		return(0);
	}
	return(1);
};

string
udp::print()
{
	char	tmp[128];
	string ret = "UDP:";

	switch(this->family())
	{
	case AF_INET:
		ret += "AF_INET ";
		break;
	case AF_INET6:
		ret += "AF_INET6 ";
		break;
	default:
		return("OTHER");
	}
	ret += this->straddr();
	sprintf(tmp, " PORT=%" PRIu16 " FD=%d", this->port(), this->_fd);
	ret += tmp;
	return(ret);
}

udp*
peers::match(ip &a)
{
	string	s;
	sarwin::screen	scr;

	for(std::list<udp>::iterator i = _peers.begin(); i != _peers.end(); i++)
	{
		if (
			(i->family() == AF_INET && i->addr4() == a.addr4()) 
			||
			(i->family() == AF_INET6 && i->addr6() == a.addr6())
		)
		{
			s = "Found Peer Match for" + i->print();
			scr.debug(3, s);
			return(&(*i));	
		}
		s = a.print() + " not peer match for " + i->print();
		scr.debug(3, s);
	}
	s = "No Peer match found for " + a.print();
	scr.error(s);
	return(NULL);
};
			
}; // namespace sarnet

/*
 * $Log: ip.cpp,v $
 * Revision 1.2  2015/01/09 22:09:49  charlesetsmith
 * Moved all Screen output to use curses. There are two windows, the top is status
 * and error messages. THe bottom is command line interaction. Also created
 * separate name spaces for curses (sarwin) and ip (sarip) so they can be
 * independant of the base saratoga code set. Fixed error in SConstruct
 * so that checksums now compiles properly and the checksums SConstruct
 * g++ options are same as the base code set. Huge number of changes but
 * the code set now operational. Remeber you have to "sudo ./saratoga"
 * as multicast requires you to be root to run.
 *
 * Revision 1.1.1.1  2014/10/15 11:55:35  lloydwood
 * Creating saratoga-vallona-dev.
 *
 * This is the Saratoga transfer protocol Vallona implementation,
 * under development by Charles Smith.
 *
 * Revision 1.1  2014/10/04 03:09:31  chas
 * Moved all IP related class and routines from ipsock to ip.cpp * ip.h
 * as there is a classto handle ip and a separate class to
 * handle udp now.
 *
 */

