/*
 * $Id: offsetstr.cpp,v 1.2 2015/01/09 22:09:49 charlesetsmith Exp $
 */

/*
 
 Copyright (c) 2011, Charles Smith
 All rights reserved.

 Redistribution and use in source and binary forms, with or without modification, 
 are permitted provided that the following conditions are met:

    * Redistributions of source code must retain the above copyright notice, this 
      list of conditions and the following disclaimer.
    * Redistributions in binary form must reproduce the above copyright notice, this
      list of conditions and the following disclaimer in the documentation and/or 
      other materials provided with the distribution.
    * Neither the name of Vallona Networks nor the names of its contributors 
      may be used to endorse or promote products derived from this software without 
      specific prior written permission.

 THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND 
 ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED 
 WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
 IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, 
 INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, 
 BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, 
 DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF 
 LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE 
 OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED 
 OF THE POSSIBILITY OF SUCH DAMAGE.

 */

#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>

#include "saratoga.h"
#include "sarflags.h"

using namespace std;

namespace saratoga
{

// Test out the checksum logic and == and != checking
string
offsetstr(f_descriptor d, offset_t o)
{
	char	tmp[128];
	uint16_t tmp16 = (uint16_t) o;
	uint32_t tmp32 = (uint32_t) o;
	uint64_t tmp64 = (uint64_t) o;
#ifdef UINT128_T
	uint128_t tmp128 = (uint128_t) o;
#endif

	string s = "";
	switch(d)
	{
	case F_DESCRIPTOR_16:
		sprintf(tmp, "%" PRIu16 "", (uint16_t) tmp16);
		break;
	case F_DESCRIPTOR_32:
		sprintf(tmp, "%" PRIu32 "", (uint32_t) tmp32);
		break;
	case F_DESCRIPTOR_64:
		sprintf(tmp, "%" PRIu64 "", (uint64_t) tmp64);
		break;
	case F_DESCRIPTOR_128:
#ifdef UINT128_T
		sprintf(tmp, "%" PRIu128 "", (uint128_t) tmp128);
#else
		sprintf(tmp, "128 bit Offset not supported");
#endif
		break;
	}
	s += tmp;
	return s;
}

}; // Namespace saratoga

/*
 * $Log: offsetstr.cpp,v $
 * Revision 1.2  2015/01/09 22:09:49  charlesetsmith
 * Moved all Screen output to use curses. There are two windows, the top is status
 * and error messages. THe bottom is command line interaction. Also created
 * separate name spaces for curses (sarwin) and ip (sarip) so they can be
 * independant of the base saratoga code set. Fixed error in SConstruct
 * so that checksums now compiles properly and the checksums SConstruct
 * g++ options are same as the base code set. Huge number of changes but
 * the code set now operational. Remeber you have to "sudo ./saratoga"
 * as multicast requires you to be root to run.
 *
 * Revision 1.1.1.1  2014/10/15 11:55:35  lloydwood
 * Creating saratoga-vallona-dev.
 *
 * This is the Saratoga transfer protocol Vallona implementation,
 * under development by Charles Smith.
 *
 * Revision 1.3  2014/10/04 03:04:37  chas
 * Just too much to explain all. We are moving to using ncurses for terminal I/O
 * We have the basis for the main saratoga loop going
 * Beacon NEARLY works
 * Lots of command line additions (hence the need to get ncurses going. THere
 * is just WAY TOO MUCH screen stuff/debug going on. So Command line is
 * in the bottom window and Error/Status Outputs are in the top window.
 *
 * Revision 1.2  2013/12/02 07:13:48  chas
 * *** empty log message ***
 *
 * Revision 1.1  2013/05/30 23:12:58  chas
 * Added offsetstr.cpp to the code tree
 *
 */

