/*
 * $Id: saratoga.cpp,v 1.4 2015/01/10 11:46:18 charlesetsmith Exp $
 */
#include <cstring>
#include <string>
#include <vector>
#include <sys/types.h>
#include <sys/select.h>
#include <ctype.h>
#include "screen.h"
#include "ip.h"
#include "saratoga.h"
#include "sarflags.h"
#include "timestamp.h"
#include "cli.h"
#include "globals.h"

#include "frame.h"
#include "beacon.h"
#include "data.h"
// #include "status.h"
// #include "metadata.h"

#define CTRL_C	3
#define CTRL_D	4

int	globfred = 1;


using namespace std;

namespace saratoga
{

// Saratoga well known port
#define SARATOGA	7542

sarnet::udp*	v4in;
sarnet::udp*	v6in;
sarnet::udp*	v4mcastout;
sarnet::udp*	v6mcastout;
sarnet::udp*	v4mcastin;
sarnet::udp*	v6mcastin;
sarnet::udp*	v4loop;
sarnet::udp*	v6loop;

/*
 * These defined in globals.cpp
 * as they are used inside saratoga library
 * v4out, v6out, sarpeers
 */

// Work out what frame type we have read and handle it
void
readhandler(char *buf, size_t len)
{
	sarwin::screen	scr;
//	extern string printsarflags(flag_t);

	flag_t	flags;
	frame	*f;
// 	status *s;
//	request *r;
//	metadata *m;

	if (len < 0)
	{
		scr.error("Bad Frame: len -1");
		return;
	}
	if (len == 0)
	{
		scr.error("Bad Frame: len 0");
		return;
	}
	if (len < sizeof(flag_t))
	{
		scr.error("Bad Frame: Len < 4 so not a saratoga frame");
		return;
	}

	memcpy(&flags, buf, sizeof(flag_t));
	flags = (flag_t) ntohl(flags);
//	cerr << "Read Handler";
//	cerr << printsarflags(flags) << endl;

	Fversion	version(flags);
	Fframetype	frametype(flags);
	Fdescriptor	descriptor(flags);

	if (version.get() != F_VERSION_1)
	{
		scr.error("Bad Frame: Not Saratoga Version 1");
		return;
	}
	switch(frametype.get())
	{
	case F_FRAMETYPE_BEACON:
		scr.debug(1, "We have received a Beacon!");
		f = new beacon(buf, len);
		scr.debug(2, f->print());
		delete f;
		break;
//	case F_FRAMETYPE_REQUEST:
//		scr.debug(1,"We have received a Request!");
//		r = new request(buf, len);
//		scr.debug(2, r->print());
//		delete r;
//		break;
//	case F_FRAMETYPE_METADATA:
//		scr.debug(1, "We have received a Metatdata!");
//		m = new metadata(buf, len);
//		scr.debug(2, m->print());
//		delete m;
//		break;
	case F_FRAMETYPE_DATA:
		scr.debug(1, "We have a received a Data!");
		f = new data(buf, len);
		scr.debug(2, f->print());
		delete f;
		break;
 //	case F_FRAMETYPE_STATUS:
 //		scr.debug(1, "We have received a Status!");
 //		s = new status(buf, len);
 //		scr.debug(2, s->print());
 //		delete s;
 //		break;
	default:
		scr.debug(1, "Bad Frame: Invalid Frame Type");
		break;
	}
}

// The command line interface flag classes
// these flags and variables are used to set the protocol flags
// default values and running state
// We have a class for each major cli keyword
// The handler is cmd::cmd_keyword
// The cli flags are cli_keyword

}; // nmaespace saratoga

// *************************************************************************************

using namespace saratoga;

// Split a string into a vector of strings
// delimiter between is white space
void
splitargs(string s, std::vector<string> &x)
{
	string curarg = "";
	char	prevc = ' ';

	// Get rid of what was there previously
	if (x.size() > 0)		
		x.erase(x.begin(), x.end());
	for (size_t i = 0; i < s.size(); i++)
	{
		if (s[i] == ' ' || s[i] == '\t')
		{
			if (prevc == ' ' || prevc == '\t')
				continue;
			if (curarg != "")
			{
				x.push_back(curarg);
				curarg = "";
			}
		}
		else
		{
			prevc = s[i];
			curarg += s[i];
		}
	}
	if (curarg != "")
		x.push_back(curarg);
}

int 
maxfd()
{
	int	largest = -1;

#define fdmax(x, y) ((x) > (y) ? (x) : (y))

	// largest = fdmax(largest, STDIN_FILENO);
	largest = fdmax(largest, STDOUT_FILENO);
	largest = fdmax(largest, STDERR_FILENO);
	largest = fdmax(largest, v4in->fd());
	largest = fdmax(largest, v6in->fd());
	largest = fdmax(largest, v4mcastout->fd());
	largest = fdmax(largest, v6mcastout->fd());
	largest = fdmax(largest, v4mcastin->fd());
	largest = fdmax(largest, v6mcastin->fd());
	largest = fdmax(largest, v4loop->fd());
	largest = fdmax(largest, v6loop->fd());
	largest = fdmax(largest, v4out->fd());
	largest = fdmax(largest, v6out->fd());
	return(largest);
}

#define MAXFRAME 1500

int 
main(int argc, char **argv)
{
	sarwin::screen	scr;
	string		errstr;
	string args;
	std::vector<string> arglist;
	fd_set	crfd;
	fd_set	cwfd;
	cmds c;
	struct timespec		wakeup;
	int	inputch = 0;
	int	selval;		// Value returned by pselect
	char	buf[9000];	// Current frame input buffer
	int	sz;		// Current frame size

	// Initialise curses - Two windows top 	and bottom
	sarwin::initwindows();

	sarnet::netifaces	interfaces;
	scr.msg(interfaces.print());
	sarnet::netiface *firstv4iface = interfaces.first(AF_INET);
	sarnet::ip	*fv4 = new sarnet::ip(firstv4iface->ifip());
	sarnet::netiface *firstv6iface = interfaces.first(AF_INET6);
	sarnet::ip	*fv6 = new sarnet::ip(firstv6iface->ifip());
	errstr = "First IPv4 is:" + fv4->print();
	scr.msg(errstr);
	errstr = "First IPv6 is:" + fv6->print();
	scr.msg(errstr);

	// IPv4 listening input socket
	v4in = new sarnet::udp(AF_INET, SARATOGA);
	if (v4in->fd() < 0)
		scr.fatal("Can't bind ipv4 saratoga socket for listening\n");
	// IPv6 listening input socket
	v6in = new sarnet::udp(AF_INET6, SARATOGA);
	if (v6in->fd() < 0)
		scr.fatal("Can't bind ipv6 saratoga socket for listening\n");
	// IPv4 output socket we write to
	v4out = new	sarnet::udp(fv4->print(), SARATOGA);
	if (v4out->fd() < 0)
		scr.fatal("Can't bind to ipv4 saratoga local ip\n");
	// IPv6 output socket we write to
	v6out = new	sarnet::udp(fv6->print(), SARATOGA);
	if (v6out->fd() < 0)
		scr.fatal("Can't bind to ipv6 saratoga local ip\n");
	// IPv4 loopback we can write to
	v4loop = new	sarnet::udp("127.0.0.1", SARATOGA);
	if (v4loop->fd() < 0)
		scr.fatal("Can't bind to ipv4 saratoga loopback ip\n");
	// IPv6 loopback we can write to
	v6loop  = new	sarnet::udp("::1", SARATOGA);
	if (v6loop->fd() < 0)
		scr.fatal("Can't bind to ipv6 saratoga loopback ip\n");
	// Bind the output IPv4 multicast to the IPv4 output socket
	v4mcastout = new sarnet::udp(sarnet::MCAST_OUT, v4out, "224.0.0.108", SARATOGA);
	if (v4mcastout->fd() < 0)
		scr.fatal("Can't bind to ipv4 saratoga multicast out - has to be run in root\n");

	// Bind the output IPv6 multicast to the IPv6 output socket
	v6mcastout  = new sarnet::udp(sarnet::MCAST_OUT, v6out, "FF02:0:0:0:0:0:0:6c", SARATOGA);
	if (v6mcastout->fd() < 0)
		scr.fatal("Can't bind to ipv6 saratoga multicast out - has to be run as root\n");

	v4mcastin = new	sarnet::udp(sarnet::MCAST_IN, v4in, "224.0.0.108", SARATOGA);
	if (v4mcastin->fd() < 0)
		scr.fatal("Can't bind to ipv4 saratoga multicast in\n");
	v6mcastin  = new sarnet::udp(sarnet::MCAST_IN, v6in, "FF02:0:0:0:0:0:0:6c", SARATOGA);
	if (v6mcastin->fd() < 0)
		scr.fatal("Can't bind to ipv6 saratoga multicast in\n");

	// Print them all out
	errstr = "IPv4 Input " + v4in->print() + '\n';
	errstr += "IPv6 Input " + v6in->print() + '\n';
	errstr += "IPv4 Output " + v4out->print()  + '\n';
	errstr +=  "IPv6 Output " + v6out->print()  + '\n';
	errstr +=  "IPv4 Mcast In " + v4mcastin->print()  + '\n';
	errstr +=  "IPv6 Mcast In " + v6mcastin->print()  + '\n';
	errstr +=  "IPv4 Mcast Out " + v4mcastout->print()  + '\n';
	errstr +=  "IPv6 Mcast Out " + v6mcastout->print()  + '\n';
	errstr +=  "IPv4 Loopback " + v4loop->print()  + '\n';
	errstr +=  "IPv6 Loopback " + v6loop->print()  + '\n';
	scr.msg(errstr);
	scr.msg("Max FD is: %d", maxfd());
	sleep(5);

	wakeup.tv_sec = 1; 
	wakeup.tv_nsec = 500000; // Wake up every 1.5 second in select
	
	// Prompt
	scr.prompt("saratoga: ");
	int sent = 0;
	FD_ZERO(&crfd);
	FD_ZERO(&cwfd);
	// Get set to write

mainloop:
	while (1)
	{
		scr.debug(3, "Around again");
		// Get set to read
		FD_ZERO(&crfd);
		FD_SET(STDIN_FILENO, &crfd);
		FD_SET(v4in->fd(), &crfd);
		FD_SET(v6in->fd(), &crfd);
		FD_SET(v4mcastin->fd(), &crfd);
		FD_SET(v6mcastin->fd(), &crfd);

		selval = pselect(maxfd() + 1, &crfd, &cwfd, NULL, &wakeup, NULL);
		switch(selval)
		{
		case -1:
			scr.perror(errno, "Failure in select");
			scr.fatal("Select Failed");
		case 0: // Timeout expired just go around again
			scr.debug(3, "Select Timeout");
			break;
		default:
			break;
		}

		if (saratoga::c_beacon.canrun() && saratoga::c_beacon.timerexpired())
		{
			scr.debug(1, "Sending a beacon");
			saratoga::c_beacon.execute();
		}
		if (saratoga::c_exit.flag() >= 0)
		{
			scr.msg("Exit Flag Set Bye!!");
			goto closedown;
		}
		
		// CLI Inputs
		if (FD_ISSET(STDIN_FILENO, &crfd))
		{
			// inputch = inch();
			// We have to use getchar here and not inch as select only works properly with getchar
			// if you try and use inch it does not set the FD and spins the loop with select not catching it
			// So the design is - Use curses for all output and standard system / library for input
			char inchar = getchar();
			inputch = (int) inchar;
			// inputch = scr.readch();
			if (isprint(inputch))
			{
				scr.debug(4, "Read Character:<%c>", inchar);
			}
			switch(inputch)
			{
			case '\r':
			case '\n':
				scr.std('\n');
				splitargs(args, arglist);
				if (arglist.size() < 1)
				{
					scr.prompt("saratoga: ");
					args = "";
					goto mainloop;
				}
				// Run the command with the arglist
				if (c.cmdstr(arglist[0]) == "" || !(c.runcmd(arglist)))
				{
					string instr = arglist[0] + ": invalid command";
					scr.info(instr);
					scr.prompt("saratoga: ");
					args = "";
					goto mainloop;
				}
				args = "";
				scr.prompt("saratoga: ");
				break;
			case CTRL_C:
			case CTRL_D:
				scr.msg("Read EOF Goodbye!!");
				goto closedown;
			case '\t':
				inputch = ' '; // Tabs become spaces and Yes we want to fall through
			case '.':
			case ' ':
			case '?':
			default:
				if (isprint(inputch))
				{
					args += (char) inputch;
					scr.std(inputch);
				}
				else
					scr.error("Invalid character <%d> entered", inputch);
			}
		}
		
		// Handle V4 Input frames
		if (FD_ISSET(v4in->fd(), &crfd))
		{
			sz = v4in->rx(buf);
			scr.msg("v4in Read %d bytes", sz);
			saratoga::readhandler(buf, sz);
		}
		if (FD_ISSET(v4mcastin->fd(), &crfd))
		{
			// Handle V4 Input Multicast frames
			sz = v4mcastin->rx(buf);
			scr.msg("v4mcastin Read %d bytes", sz);
			saratoga::readhandler(buf, sz);
		}
		if (FD_ISSET(v6in->fd(), &crfd))
		{
			// Handle V6 Input frames
			sz = v6in->rx(buf);
			scr.msg("v6in Read %d bytes");
			saratoga::readhandler(buf, sz);
		}
		if (FD_ISSET(v6mcastin->fd(), &crfd))
		{
			// Handle V6 Input Multicast frames
			sz = v6mcastin->rx(buf);
			scr.msg("v6mcastin Read %d bytes", sz);
			saratoga::readhandler(buf, sz);
		}

		// Outputs
		(v4mcastout->ready()) ?  FD_SET(v4mcastout->fd(), &cwfd) :
			FD_CLR(v4mcastout->fd(), &cwfd);
		if (FD_ISSET(v4mcastout->fd(), &cwfd))
		{
			// Handle V4 Multicast Output frames
			if ((sz = v4mcastout->tx()) > 0)
				scr.msg("v4mcastout Wrote %d bytes", sz);
		}

		(v6mcastout->ready()) ?  FD_SET(v6mcastout->fd(), &cwfd) :
				FD_CLR(v6mcastout->fd(), &cwfd);

		if (FD_ISSET(v6mcastout->fd(), &cwfd))
		{
			// Handle V6 Multicast Output frames
			if ((sz =  v6mcastout->tx()) > 0)
				scr.msg("v6mcastout Wrote %d bytes", sz);
		}

		(v4loop->ready()) ?  FD_SET(v4loop->fd(), &cwfd) :
				FD_CLR(v4loop->fd(), &cwfd);
		if (FD_ISSET(v4loop->fd(), &cwfd))
		{
			// Handle V4 Input frames
			if ((sz = v4loop->tx()) > 0)
				scr.msg("v4loop Wrote %d bytes", sz);
		}

		(v6loop->ready()) ?  FD_SET(v6loop->fd(), &cwfd) :
				FD_CLR(v6loop->fd(), &cwfd);
		if (FD_ISSET(v6loop->fd(), &cwfd))
		{
			// Handle V6 Local Output frames
			if ((sz = v6loop->tx()) > 0)
				scr.msg("v6loop Wrote %d bytes", sz);
		}

		(v4out->ready()) ?  FD_SET(v4out->fd(), &cwfd) :
				FD_CLR(v4out->fd(), &cwfd);
		if (FD_ISSET(v4out->fd(), &cwfd))
		{
			// Handle V6 Local Output frames
			if ((sz = v4out->tx()) > 0)
				scr.msg("v4out Wrote %d bytes", sz);
		}

		(v6out->ready()) ?  FD_SET(v6out->fd(), &cwfd) :
				FD_CLR(v6out->fd(), &cwfd);
		if (FD_ISSET(v6out->fd(), &cwfd))
		{
			// Handle V6 Local Output frames
			if ((sz = v6out->tx()) > 0)
				scr.msg("v6out Wrote %d bytes", sz);
		}

		// Send frames out to our peers as required
		for (std::list<sarnet::udp>::iterator i = sarpeers.begin(); i != sarpeers.end(); i++)
		{
			string s = i->print();	
			(i->ready()) ? FD_SET(i->fd(), &cwfd) :
				FD_CLR(i->fd(), &cwfd);
			if (FD_ISSET(i->fd(), &cwfd))
			{
				if ((sz = i->tx()) > 0)
				{
					scr.debug(2, "Wrote %d bytes to %s", sz, s.c_str());
				}
			}
			else
				scr.debug(2, "Nothing written to %s", s.c_str());
		}

		if (sent) ;
	}
closedown:
	// CLose up curses
	sarwin::endwindows();
	
	cerr << endl << "Bye!!" << endl;
	FD_CLR(v6out->fd(), &cwfd);
	FD_CLR(v4out->fd(), &cwfd);
	FD_CLR(v6loop->fd(), &cwfd);
	FD_CLR(v4loop->fd(), &cwfd);
	FD_CLR(v6mcastout->fd(), &cwfd);
	FD_CLR(v4mcastout->fd(), &cwfd);
	FD_CLR(v6in->fd(), &crfd);
	FD_CLR(v4in->fd(), &crfd);
	FD_CLR(v6mcastin->fd(), &crfd);
	FD_CLR(v4mcastin->fd(), &crfd);
	return(saratoga::c_exit.flag());
}

/*
 * $Log: saratoga.cpp,v $
 * Revision 1.4  2015/01/10 11:46:18  charlesetsmith
 * Bit more of an explanation on running as root. Remember you have to run
 * saratoga as the superuser in order to write multicast output packets in ubuntu
 *
 * Revision 1.3  2015/01/10 09:16:24  charlesetsmith
 * Problem with CVS repository - Pointed to the worng one.
 *
 * Revision 1.2  2015/01/09 22:09:49  charlesetsmith
 * Moved all Screen output to use curses. There are two windows, the top is status
 * and error messages. THe bottom is command line interaction. Also created
 * separate name spaces for curses (sarwin) and ip (sarip) so they can be
 * independant of the base saratoga code set. Fixed error in SConstruct
 * so that checksums now compiles properly and the checksums SConstruct
 * g++ options are same as the base code set. Huge number of changes but
 * the code set now operational. Remeber you have to "sudo ./saratoga"
 * as multicast requires you to be root to run.
 *
 * Revision 1.1.1.1  2014/10/15 11:55:35  lloydwood
 * Creating saratoga-vallona-dev.
 *
 * This is the Saratoga transfer protocol Vallona implementation,
 * under development by Charles Smith.
 *
 */

