/*
 * $Id: saratoga.h,v 1.2 2015/01/09 22:09:49 charlesetsmith Exp $
 */

/*
 
 Copyright (c) 2011, Charles Smith
 All rights reserved.

 Redistribution and use in source and binary forms, with or without modification, 
 are permitted provided that the following conditions are met:

    * Redistributions of source code must retain the above copyright notice, this 
      list of conditions and the following disclaimer.
    * Redistributions in binary form must reproduce the above copyright notice, this
      list of conditions and the following disclaimer in the documentation and/or 
      other materials provided with the distribution.
    * Neither the name of Vallona Networks nor the names of its contributors 
      may be used to endorse or promote products derived from this software without 
      specific prior written permission.

 THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND 
 ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED 
 WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
 IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, 
 INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, 
 BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, 
 DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF 
 LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE 
 OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED 
 OF THE POSSIBILITY OF SUCH DAMAGE.

 */

#ifndef _SARATOGA_H
#define _SARATOGA_H

#ifndef __STDC_FORMAT_MACROS
#define __STDC_FORMAT_MACROS	// So we can d PRI_u64 in printf
#endif

#include <inttypes.h>
#include <arpa/inet.h>
#include <string>

/* 
 * At this stage we do not support 128 bit descriptors but the code is
 * all there. Just define this and we are good to go
 */
#undef UINT128_T

#ifdef UINT128_T
extern uint128_t		htonlll(uint128_t);
extern uint128_t		ntohlll(uint128_t);
#endif // UINT128_T

extern uint64_t			htonll(uint64_t);
extern uint64_t			ntohll(uint64_t);

extern int			debuglevel();

namespace saratoga 
{

typedef uint32_t		flag_t;		// Saratoga Flags
typedef uint16_t		dflag_t;	// Directory Flags
typedef uint32_t		dtime_t;	// Directory Time Fields
typedef unsigned char	tflag_t;	// Timestamp Flags
typedef uint32_t		session_t;	// Session ID
// typedef off_t			offset_t;	// Memory & Offset maths I need it to be the biggest unsigned int type
typedef off64_t		offset_t;	// Memory & Offset maths I need it to be the biggest unsigned int type



// enum boolean	{ TRUE = 1, FALSE = 0 }; // Good old bools...

// hton and ntoh conversions
typedef uint16_t		hton_t;
typedef uint16_t		ntoh_t;
typedef uint32_t		htonl_t;
typedef uint32_t		ntohl_t;
typedef uint64_t		htonll_t;
typedef uint64_t		ntohll_t;
#ifdef UINT128_T
typedef uint128_t		htonlll_t;	// htonlll conversions - maybe one day :)
typedef uint128_t		ntohlll_t;	// ntohlll conversions
#endif // UINT128_T

typedef unsigned int 		uint_t;		// Used in sprintf's

#define SARATOGASOCKET		7542

extern int	global_debug;

} // namespace saratoga

#endif // _SARATOGA_H


/*
 * $Log: saratoga.h,v $
 * Revision 1.2  2015/01/09 22:09:49  charlesetsmith
 * Moved all Screen output to use curses. There are two windows, the top is status
 * and error messages. THe bottom is command line interaction. Also created
 * separate name spaces for curses (sarwin) and ip (sarip) so they can be
 * independant of the base saratoga code set. Fixed error in SConstruct
 * so that checksums now compiles properly and the checksums SConstruct
 * g++ options are same as the base code set. Huge number of changes but
 * the code set now operational. Remeber you have to "sudo ./saratoga"
 * as multicast requires you to be root to run.
 *
 * Revision 1.1.1.1  2014/10/15 11:55:35  lloydwood
 * Creating saratoga-vallona-dev.
 *
 * This is the Saratoga transfer protocol Vallona implementation,
 * under development by Charles Smith.
 *
 * Revision 1.8  2014/10/04 03:04:37  chas
 * Just too much to explain all. We are moving to using ncurses for terminal I/O
 * We have the basis for the main saratoga loop going
 * Beacon NEARLY works
 * Lots of command line additions (hence the need to get ncurses going. THere
 * is just WAY TOO MUCH screen stuff/debug going on. So Command line is
 * in the bottom window and Error/Status Outputs are in the top window.
 *
 * Revision 1.7  2013/10/04 02:04:18  chas
 * Finally fixed the damn issues with METADATA. Problem was that I was not
 * updating the Frame copy of the metadata. i.e. I needed to copy
 * to _payload and _paylen in the creator for metadata.
 *
 * Revision 1.6  2013/09/28 20:19:03  chas
 * Moved directory entry flags to separate include file
 *
 * Revision 1.5  2013/06/08 07:31:50  chas
 * Direcory entry as Gordon Ramsey would say DONE!! So it can read directory entries and get the file type and info to be ready to go into a metadata or read from to get the info.
 *
 * Revision 1.4  2013/06/05 07:20:09  chas
 * Some issues with geting the descriptor size. It is now called descriptor.length(). Also conversion hnton and ntoh issues. Is all consistant now. Beacons good, data good. Lets move on to metadata.
 *
 * Revision 1.3  2013/03/23 12:01:00  chas
 * Finally sorted out polymorhic "frame" class and beacon all done
 * so now I can move onto similar for other frame types
 *
 * Revision 1.2  2013/03/15 04:54:35  chas
 * Moving frames to polymophism so we have a single base class of frame and
 * then polymorph frame types of beacon and data. As we move forward
 * I will add the other saratoga frame types. Lots more work to do here yet
 * as I havn't got the base class sorted yet.
 *
 * Revision 1.1.1.1  2013/03/09 07:35:04  chas
 * Saratoga C++ Primary Implementation
 * Charles Smith charlesetsmith@me.com
 *
 */
