/*
 * $Id: sarflags.cpp,v 1.2 2015/01/09 22:09:49 charlesetsmith Exp $
 */

/*
 
 Copyright (c) 2012, Charles Smith
 All rights reserved.

 Redistribution and use in source and binary forms, with or without modification, 
 are permitted provided that the following conditions are met:

    * Redistributions of source code must retain the above copyright notice, this 
      list of conditions and the following disclaimer.
    * Redistributions in binary form must reproduce the above copyright notice, this
      list of conditions and the following disclaimer in the documentation and/or 
      other materials provided with the distribution.
    * Neither the name of Vallona Networks nor the names of its contributors 
      may be used to endorse or promote products derived from this software without 
      specific prior written permission.

 THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND 
 ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED 
 WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
 IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, 
 INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, 
 BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, 
 DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF 
 LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE 
 OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED 
 OF THE POSSIBILITY OF SUCH DAMAGE.

 */


#include <iostream>
#include <cstring>
#include <string>
#include <sys/time.h>
#include <sys/resource.h>

#include "saratoga.h"
#include "sarflags.h"
#include "screen.h"

using namespace std;

namespace saratoga
{

string
printsarflags(flag_t f)
{
	string s("");

	s += "\n    SARATOGA FLAGS       1                   2                   3\n";
	s += "    |0 1 2 3 4 5 6 7|8 9 0 1 2 3 4 5|6 7 8 9 0 1 2 3|4 5 6 7 8 9 0 1|\n";
	s += "    |";
	for (int i = 0; i < 32; i++)
	{
		flag_t tmpf = f;
		tmpf <<= i;
		tmpf >>= 31;
		if (tmpf == 1)
			s += "1|";
		else
			s += " |";
	}
	s += "\n";
	return s;
}

// What is the descriptor size for the largest file type
enum f_descriptor
maxdescriptor()
{
	sarwin::screen	msg;
	struct rlimit rlim;

	if (getrlimit(RLIMIT_FSIZE, &rlim) < 0)
	{
		msg.perror(errno, "maxdescriptor(): Cannot getrlimit setting 16 bit:");
		return(F_DESCRIPTOR_16);
	}
	if (rlim.rlim_cur <= 65536)	// 2^16
		return(F_DESCRIPTOR_16);
	if (rlim.rlim_cur <= 4294967296) // 2^32
		return(F_DESCRIPTOR_32);
	return(F_DESCRIPTOR_64);	// 2^64 or more we dont do 2^128 yet
}

}; // Namespace saratoga

/*
 * $Log: sarflags.cpp,v $
 * Revision 1.2  2015/01/09 22:09:49  charlesetsmith
 * Moved all Screen output to use curses. There are two windows, the top is status
 * and error messages. THe bottom is command line interaction. Also created
 * separate name spaces for curses (sarwin) and ip (sarip) so they can be
 * independant of the base saratoga code set. Fixed error in SConstruct
 * so that checksums now compiles properly and the checksums SConstruct
 * g++ options are same as the base code set. Huge number of changes but
 * the code set now operational. Remeber you have to "sudo ./saratoga"
 * as multicast requires you to be root to run.
 *
 * Revision 1.1.1.1  2014/10/15 11:55:35  lloydwood
 * Creating saratoga-vallona-dev.
 *
 * This is the Saratoga transfer protocol Vallona implementation,
 * under development by Charles Smith.
 *
 */

