/*
 * $Id: screen.h,v 1.1 2015/01/09 22:17:48 charlesetsmith Exp $
 */

/*
 
 Copyright (c) 2011, Charles Smith
 All rights reserved.

 Redistribution and use in source and binary forms, with or without modification, 
 are permitted provided that the following conditions are met:

    * Redistributions of source code must retain the above copyright notice, this 
      list of conditions and the following disclaimer.
    * Redistributions in binary form must reproduce the above copyright notice, this
      list of conditions and the following disclaimer in the documentation and/or 
      other materials provided with the distribution.
    * Neither the name of Vallona Networks nor the names of its contributors 
      may be used to endorse or promote products derived from this software without 
      specific prior written permission.

 THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND 
 ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED 
 WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
 IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, 
 INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, 
 BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, 
 DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF 
 LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE 
 OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED 
 OF THE POSSIBILITY OF SUCH DAMAGE.

 */

#ifndef _SCREEN_H
#define _SCREEN_H

#include <stdio.h>
#include <string>
#include <errno.h>
#include <ncurses.h>
#include "saratoga.h"
#include "cli.h"

using namespace std;
using namespace saratoga;

// The color pairs
static const short RED_BLACK		= 1;
static const short GREEN_BLACK		= 2;
static const short YELLOW_BLACK		= 3;
static const short BLUE_BLACK		= 4;
static const short MAGENTA_BLACK	= 5;
static const short CYAN_BLACK		= 6;
static const short WHITE_BLACK		= 7;
static const short BLACK_RED		= 8;
static const short BLACK_GREEN		= 9;
static const short BLACK_YELLOW		= 10;
static const short BLACK_BLUE		= 11;
static const short BLACK_MAGENTA	= 12;
static const short BLACK_CYAN		= 13;
static const short BLACK_WHITE		= 14;

/*
 * Handle debug and error output.
 * Flexible as you can set the level and also what file you want to talk to
 * defaults are stderr of course. When setting a new fp make sure we dont close stdout and stderr
 */

namespace sarwin
{

extern void initwindows();
extern void endwindows();

class screen
{
private:
	FILE	*_fp;
public:
	screen()	{ _fp = stderr; };
	screen(string outfile) { this->setout(outfile); };
	~screen()	{ if (_fp != NULL && _fp != stderr && _fp != stdout) fclose(_fp); };
	int		readch();
	void		std(char c);
	void		std(string fmt, ...);
	void		prompt(string fmt, ...);
	void		info(string fmt, ...);
	void		msg(string fmt, ...);
	void		debug(int lev, string fmt, ...);
	void		error(string fmt, ...);
	void		warning(string fmt, ...);
	void		perror(int err, string fmt, ...);
	void		fatal(string fmt, ...);
	void		setout(string outfile);
	void		setout() { if (_fp != NULL && _fp != stderr && _fp != stdout) fclose(_fp); _fp = stderr; };
};

}; // namespace sarwin

#endif // _SCREEN_H

/*
 * $Log: screen.h,v $
 * Revision 1.1  2015/01/09 22:17:48  charlesetsmith
 * Screen curses code for all output. We have two windows, the top one is used for
 * error and log messaging and the bottom is used for command line interaction.
 * No you cna;t use curses "getch" or "wgetch" when you are polling for input
 * using a select() call (as we do here in saratoga). So I am using good old
 * getchar(). THis is OK given that all of the user input is always in the
 * bottom window. Actually moving forward I don;t know if this can be solved.
 * So anyway ALL of the curses dependencies are in the "screen.cpp" and "screen.h"
 * files so moving over to a separate windowing or gui system is just going to
 * be a matter of fiddling the finctions in this. I have also put them into
 * a separate namespace "sarwin". THe ONLY interaction between "sarwin" and
 * "saratoga" variables is the c_debug. Which is the global class variable that
 * sets the debug level. You can adjust the debug level to increase/decrease the
 * amount of info coming to you in the log window.
 *
 * 
 * Revision 1.1.1.1  2014/10/15 11:55:35  lloydwood
 * Creating saratoga-vallona-dev.
 *
 * This is the Saratoga transfer protocol Vallona implementation,
 * under development by Charles Smith.
 *
 * Revision 1.3  2014/10/04 03:04:36  chas
 * Just too much to explain all. We are moving to using ncurses for terminal I/O
 * We have the basis for the main saratoga loop going
 * Beacon NEARLY works
 * Lots of command line additions (hence the need to get ncurses going. THere
 * is just WAY TOO MUCH screen stuff/debug going on. So Command line is
 * in the bottom window and Error/Status Outputs are in the top window.
 *
 * Revision 1.2  2013/04/01 09:57:55  chas
 * Working on dirent. More work to do yet in working out a default
 * descriptor size based upon the size of the incoming file. This
 * may be overridden once we have a session defined. stat call
 * is nearly done.
 *
 * Revision 1.1.1.1  2013/03/09 07:35:04  chas
 * Saratoga C++ Primary Implementation
 * Charles Smith charlesetsmith@me.com
 *
 * Revision 1.1  2011-01-18 10:15:43  chas
 * Refining logging and error printing classes
 *
 */

