/*
 * $Id: session.h,v 1.2 2015/01/09 22:09:49 charlesetsmith Exp $
 */

/*
 
 Copyright (c) 2012, Charles Smith
 All rights reserved.

 Redistribution and use in source and binary forms, with or without modification, 
 are permitted provided that the following conditions are met:

    * Redistributions of source code must retain the above copyright notice, this 
      list of conditions and the following disclaimer.
    * Redistributions in binary form must reproduce the above copyright notice, this
      list of conditions and the following disclaimer in the documentation and/or 
      other materials provided with the distribution.
    * Neither the name of Vallona Networks nor the names of its contributors 
      may be used to endorse or promote products derived from this software without 
      specific prior written permission.

 THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND 
 ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED 
 WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
 IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, 
 INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, 
 BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, 
 DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF 
 LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE 
 OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED 
 OF THE POSSIBILITY OF SUCH DAMAGE.

 */

/*
 * Class Handling Storage for a saratoga Session
 * We have the flags applicable to across a session here along
 * with the lined lists doing I/O buffering
 */

/*
 **********************************************************************************************************
 */

#ifndef _SESSION_H
#define _SESSION_H

#include <string>
using namespace std;

#include "saratoga.h"
#include "sarflags.h"
#include "screen.h"

namespace saratoga

{

/*
 **********************************************************************************************************
 */

class session
{
protected:
	session_t		_session; // Unique Identifier of this session
	/*
	 * These are the flags that are applicable across a whole session
	 * SO they can be set once then passed along with the Session class to assemble 
	 * each frame type they apply to rather than setting all the flags all the time 
	 * which would be both painfull and very error prone.
	 */
	Fversion		_version;		// Our Implementaiton only does V1
	Fdescriptor		_descriptor;		// What is the descriptor for this session
	Fbundle			_bundle;		// We dont do bundles on our version
	Fstream			_stream;		// Yes we do support streams
	Ftransfer		_transfer;		// What are we transferring ?	
	Ftimestamp		_timestamp;		// Do we add timestamps in on the transfer frames ?
	Fudptype		_udptype;		// We only support UDP on our version
	Fmetadata_udptype	_metadata_udptype; 	// As above for metadata flags
	Fsessiontype		_sessiontype;		// Are we getting or deleting ?
	Ffileordir		_fileordir;		// Are we a file or directory session ?
	Ffreespace		_freespace;		// do we advertise free space ?
	Ffreespaced		_freespaced;		// What is the descriptor size of freespace ?
	Fchecksum		_checksum;		// What checksum type are we using ?
public:
};

} // namespace saratoga

#endif // _SESSION_H
