/*
 * $Id: sysinfo.cpp,v 1.2 2015/01/09 22:09:49 charlesetsmith Exp $
 */

/*

 Copyright (c) 2014, Charles Smith
 All rights reserved.

 Redistribution and use in source and binary forms, with or without modification,
 are permitted provided that the following conditions are met:

    * Redistributions of source code must retain the above copyright notice, this
      list of conditions and the following disclaimer.
    * Redistributions in binary form must reproduce the above copyright notice, this
      list of conditions and the following disclaimer in the documentation and/or
      other materials provided with the distribution.
    * Neither the name of Vallona Networks nor the names of its contributors
      may be used to endorse or promote products derived from this software without
      specific prior written permission.

 THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND
 ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
 IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT,
 INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING,
 BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
 LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE
 OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED
 OF THE POSSIBILITY OF SUCH DAMAGE.

 */

#include <cstring>
#include <string>
#include <sys/statvfs.h>
#include <sys/types.h>
#include <sys/stat.h>
#include "screen.h"
#include "sysinfo.h"

using namespace std;

namespace saratoga
{


// How much disk space is free on the named fs default bytes
offset_t
sysinfo::diskfree()
{
	return(this->diskfree(BYTE));
}

offset_t
sysinfo::diskfree(enum kmg div)
{
	sarwin::screen	msg;
	char	*fnPath = new char[_fsname.length()];
	struct statvfs fiData;
	uint64_t size;

	strcpy(fnPath, _fsname.c_str());
	if ((statvfs(fnPath, &fiData)) < 0)
	{
		msg.error("No such filesystem: %s\n", fnPath);
		delete [] fnPath;
		return(0);
	}
	else
	{
		delete [] fnPath;
//		printf("Disk %s: \n", fnPath);
//		printf("\tblock size: %lu\n", fiData.f_bsize);
//		printf("\ttotal no blocks: %lu\n", fiData.f_blocks);
//		printf("\tfree blocks: %lu\n", fiData.f_bfree);

		// So we dont blow size of uint64_t
		switch(div)
		{
		case BYTE:
			size = 1;
			break;
		case KILO:
			size = 1024;
			break;
		case MEGA:
			size = 1024 * 1024;
			break;
		case GIGA:
			size = 1024 * 1024 * 1024;
			break;
		default:
			return(0);
		}
		return ((fiData.f_bsize * fiData.f_bfree)/size);
	}
}

}; // namespace saratoga

/*
 * $Log: sysinfo.cpp,v $
 * Revision 1.2  2015/01/09 22:09:49  charlesetsmith
 * Moved all Screen output to use curses. There are two windows, the top is status
 * and error messages. THe bottom is command line interaction. Also created
 * separate name spaces for curses (sarwin) and ip (sarip) so they can be
 * independant of the base saratoga code set. Fixed error in SConstruct
 * so that checksums now compiles properly and the checksums SConstruct
 * g++ options are same as the base code set. Huge number of changes but
 * the code set now operational. Remeber you have to "sudo ./saratoga"
 * as multicast requires you to be root to run.
 *
 * Revision 1.1.1.1  2014/10/15 11:55:35  lloydwood
 * Creating saratoga-vallona-dev.
 *
 * This is the Saratoga transfer protocol Vallona implementation,
 * under development by Charles Smith.
 *
 */

