/*
 * $Id: test1.cpp,v 1.2 2015/01/09 22:09:49 charlesetsmith Exp $
 */

/*
 
 Copyright (c) 2011, Charles Smith
 All rights reserved.

 Redistribution and use in source and binary forms, with or without modification, 
 are permitted provided that the following conditions are met:

    * Redistributions of source code must retain the above copyright notice, this 
      list of conditions and the following disclaimer.
    * Redistributions in binary form must reproduce the above copyright notice, this
      list of conditions and the following disclaimer in the documentation and/or 
      other materials provided with the distribution.
    * Neither the name of Vallona Networks nor the names of its contributors 
      may be used to endorse or promote products derived from this software without 
      specific prior written permission.

 THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND 
 ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED 
 WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
 IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, 
 INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, 
 BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, 
 DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF 
 LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE 
 OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED 
 OF THE POSSIBILITY OF SUCH DAMAGE.

 */

#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>
#include <limits>
#include <curses.h>

#include "sysinfo.h"
#include "fileio.h"
#include "saratoga.h"
#include "sarflags.h"
#include "timestamp.h"
#include "checksums/checksum.h"
#include "frame.h"
#include "beacon.h"
#include "data.h"
#include "dirent.h"
#include "metadata.h"
#include "holes.h"
#include "ip.h"

// #define TESTBEACON
// #define TESTDATA
// #define TESTTIMESTAMP

// #include <iostream>

#define DEBUGFLAGS

using namespace std;

using namespace saratoga;

// Test out the checksum logic and == and != checking
void
testchecksum(string fname1, string fname2)
{	
	cout << "Doing No checksum" << endl;
	checksums::none	n1(fname1);
	cout << n1.print() << endl;

	cout << endl;

	cout << "Doing CRC32" << endl;
	checksums::crc32	c1(fname1);
	checksums::crc32 c2(fname2);
	cout << c1.print() << endl;
	cout << c2.print() << endl;
	if (c1 == c2)
		cout << "== CRC32 true" << endl;
	else
		cout << "== CRC32 false" << endl;
	if (c1 != c2)
		cout << "!= CRC32 true" << endl;
	else
		cout << "!= CRC32 false" << endl;
	if (c2 == "C6BE946C")
		cout << "== String CRC32 true" << endl;
	else
		cout << "== String CRC32 false" << endl;
	if (c2 != "C6BE946C")
		cout << "!= String CRC32 true" << endl;
	else
		cout << "!= String CRC32 false" << endl;

	cout << endl;

	cout << "Doing MD5" << endl;
	checksums::md5 m1(fname1);
	checksums::md5 m2(fname2);
	cout << m1.print() << endl;
	cout << m2.print() << endl;
	if (m1 == m2)
		cout << "== MD5 true" << endl;
	else
		cout << "== MD5 false" << endl;
	if (m1 != m2)
		cout << "!= MD5 true" << endl;
	else
		cout << "!= MD5 false" << endl;
	if (m2 == "C3882AC2 32E69CAA 8928D904 10E3B1F9")
		cout << "== String MD5 true" << endl;
	else
		cout << "== String MD5 false" << endl;
	if (m2 != "C3882AC2 32E69CAA 8928D904 10E3B1F9")
		cout << "!= String MD5 true" << endl;
	else
		cout << "!= String MD5 false" << endl;

	cout << endl;

	cout << "Doing SHA1" << endl;
	checksums::sha1	s1(fname1);
	checksums::sha1	s2(fname2);
	cout << s1.print() << endl;
	cout << s2.print() << endl;
	if (s1 == s2)
		cout << "== SHA1 true" << endl;
	else
		cout << "== SHA1 false" << endl;
	if (s1 != s2)
		cout << "!= SHA1 true" << endl;
	else
		cout << "!= SHA1 false" << endl;
	if (s2 == "E3989E28 0E43A79A E32067A0 30FA3E07 1397AB1F")
		cout << "== String SHA1 true" << endl;
	else
		cout << "== String SHA1 false" << endl;
	if (s2 != "E3989E28 0E43A79A E32067A0 30FA3E07 1397AB1F")
		cout << "!= String SHA1 true" << endl;
	else
		cout << "!= String SHA1 false" << endl;
	
}

int
main(int argc, char **argv)
{
	initscr();
	printw("Hello World !!!");
	refresh();
	getch();
	endwin();
	exit(0);

	cout << "Testing Saratoga" << endl;

	//if (argc != 0)
	//	return(1);

#ifdef TESTTIMESTAMP
	timestamp	t;
	cout <<"Default:" << t.print() << endl;

	timestamp	t_32(T_TSTAMP_32);
	cout << "t_32 " << t_32.print() << endl;

	timestamp	t_1(T_TSTAMP_32, 10, 0);
	cout << "t_1 " << t_1.print() << endl;

	timestamp	tn = t_32 + t_1;
	cout << "tn = t_32 + t_1 " << tn.print() << endl;

	timestamp	tn1 = tn + 25;
	cout << "tn1 = tn + 25 " << tn1.print() << endl;
	timestamp	tn2 = tn - 5;
	cout << "tn2 = tn - 5 " << tn2.print() << endl;
	tn2--;
	cout << "tn2-- " << tn2.print() << endl;
	--tn2;
	cout << "--tn2 " << tn2.print() << endl;

	timestamp	tn3(T_TSTAMP_32, 0, 0);
	tn3++;
	++tn3;
	cout << "Should be 2 " << tn3.print() << endl;
	(--tn3)--;
	cout << "Should be 0 " << tn3.print() << endl;

	timestamp	tn4(T_TSTAMP_32, 0, 0);
	--tn4;
	cout << "Should be ? " << tn4.print() << endl;

	timestamp	tn5(T_TSTAMP_32, 0, 0);
	cout << "Should be epoch " << tn5.print() << endl;
	tn5 += 60;
	cout << "Should be epoch +60 " << tn5.print() << endl;
	tn5 -= 60;
	cout << "Should be epoch again " << tn5.print() << endl;

	timestamp	tn6(T_TSTAMP_32, 100, 0);

	tn5 += tn6;
	cout << "Should be epoch +100 " << tn5.print() << endl;
	tn5 -= tn6;
	cout << "Should be epoch again " << tn5.print() << endl;

	

	timestamp	tn7(T_TSTAMP_32, 100, 0);
	timestamp	tn8(T_TSTAMP_32, 100, 0);
	if (tn7 == tn8)
		cout << "tn7 == tn8" << endl;
	if (tn7 != tn8)
		cout << "tn7 != tn8" << endl;
	if (tn7 <= tn8)
		cout << "tn7 <= tn8" << endl;
	if (tn7 >= tn8)
		cout << "tn7 >= tn8" << endl;
	if (tn7 > tn8)
		cout << "tn7 > tn8" << endl;
	if (tn7 < tn8)
		cout << "tn7 < tn8" << endl;

	timestamp	tn9(T_TSTAMP_64_32);
	cout << "Current Time is " << tn9.print() << endl;
	char	buf[16];
	memcpy(buf, tn9.hton(), 16);
	timestamp	tn10(buf);
	cout << "And it still is Current Time is " << tn10.print() << endl;

	timestamp	tn11 = tn10;
	cout << "Assignement And it still is Current TIme is " << tn11.print() << endl;

#endif

#define TESTCHECKSUMS
#ifdef TESTCHECKSUMS
	cout << "Checking where checksum is same" << endl;;
	string fname1 = "test.txt";
	string fname2 = fname1;
	testchecksum(fname1, fname2);

	cout << endl;

	cout << "Checking where checksum is different" << endl;;
	string fname3 = "test.txt";
	string fname4 = "test";
	testchecksum(fname3, fname4);
	
	cout << endl;
#endif

// #define TESTBEACONS
#ifdef TESTBEACON

	// A beacon with no freespace advertised
	beacon *b1p = new beacon( F_DESCRIPTOR_16, 
		F_STREAM_NO,
		F_TXWILLING_YES, 
		F_RXWILLING_YES, 
		"0123456789");
	cout << "b1p F_FREESPACE_NO: F_STREAM_NO, F_TXWILLING_YES, F_RXWILLING_YES, 0123456789" << endl << b1p->print() << endl;

	// Get the freespace data we need
	fsinfo  *fs = new fsinfo("."); // The current directory at the moment

	// A beacon with freespace advertised
	beacon *b2p = new beacon( F_DESCRIPTOR_64, 
		F_STREAM_NO,
		F_TXWILLING_YES, 
		F_RXWILLING_NO,
		fs,
		"9876543210");
	cout << "b2p F_FREESPACE_YES: F_DESCRIPTOR_64, F_STREAM_NO, F_TXWILLING_YES, F_RXWILLING_NO, 9876543210" << endl << b2p->print() << endl;

	// Another one with freespace
	beacon b3( F_DESCRIPTOR_64, 
		F_STREAM_NO,
		F_TXWILLING_YES, 
		F_RXWILLING_YES,
		fs,
		"9876501234");
	cout << "b3 F_FREESPACE_YES: F_DESCRIPTOR_64, F_STREAM_NO, F_TXWILLING_YES, F_RXWILLING_YES, 9876501234" << endl << b3.print() << endl;

	// ANother one without freespace
	beacon b4( F_DESCRIPTOR_32, 
		F_STREAM_NO,
		F_TXWILLING_YES, 
		F_RXWILLING_YES, 
		"abcdefghij");
	cout << "b4 F_FREESPACE_NO: F_DESCRIPTOR_32 F_STREAM_NO F_TXWILLING_YES F_RXWILLING_YES abcdefghij" << endl << b4.print() << endl;

	// Lets see how we go when we create a frame that way
	frame *f = new beacon(b2p->payload(), b2p->paylen());


	if ( f->type() == F_FRAMETYPE_BEACON)
		cout << "f (same as b2p): " << f->print() << endl;
	else
		cout << "WTF we should of had a beacon!" << endl;

	beacon b5(f->payload(), f->paylen());

	frame *f2 = &b5;

	cout << "f2 == b5 F_TXWILLING_YES:" << f2->print() << endl;

	b5.txwilling(F_TXWILLING_NO);
	cout << "f2 == b5 F_TXWILLING_NO: " << f2->print() << endl;


	cout << "b5 (same as b1p): " << b5.print() << endl;
#endif

// #define TESTDATAS
#ifdef TESTDATA
	char payload[] = "0123456789";


	data* d1 = new data( F_DESCRIPTOR_16, 
		F_TRANSFER_FILE,
		F_REQSTATUS_YES, 
		F_EOD_YES,
		F_TIMESTAMP_YES,
		99,
		9876,
		payload,
		strlen(payload));
	cout << "d1: F_DESC_16 F_TRANSFER_FILE F_REQSTATUS_YES F_EOD_YES F_TIMESTAMP_YES 99 9876 0123456789" << endl << d1->print() << endl << endl;

	data* d2 = new data( F_DESCRIPTOR_32, 
		F_TRANSFER_FILE,
		F_REQSTATUS_YES, 
		F_EOD_NO,
		F_TIMESTAMP_YES,
		9999,
		12345,
		payload,
		strlen(payload));
	cout << "d2: F_DESC_32 F_TRANSFER_FILE F_REQSTATUS_YES F_EOD_NO F_TIMESTAMP_YES 9999 12345 0123456789" << endl << d2->print() << endl << endl;

	data* d3 = new data( F_DESCRIPTOR_64, 
		F_TRANSFER_FILE,
		F_REQSTATUS_NO, 
		F_EOD_NO,
		F_TIMESTAMP_YES,
		55,
		8231,
		payload,
		strlen(payload));
	cout << "d3: F_DESC_64 F_TRANSFER_FILE F_REQSTATUS_NO F_EOD_NO F_TIMESTAMP_YES 55 8231 0123456789" << endl << d3->print() << endl << endl;

	data* d4 = new data(d2->payload(), d2->paylen());
	cout << "Should be same as d2: " << endl << d4->print() << endl << endl;
#endif

#ifdef FSINFO
	fsinfo *filsys = new fsinfo("/");

	cout << filsys->print() << endl;
#endif

#ifdef NUMLIMITS
	cout << "Minimum value for uint16: " << numeric_limits<uint16_t>::min() << endl;
	cout << "Maximum value for unint16: " << numeric_limits<uint16_t>::max() << endl;
	cout << "Minimum value for uint32: " << numeric_limits<uint32_t>::min() << endl;
	cout << "Maximum value for uint32: " << numeric_limits<uint32_t>::max() << endl;
	cout << "Minimum value for unint64: " << numeric_limits<uint64_t>::min() << endl;
	cout << "Maximum value for unint64: " << numeric_limits<uint64_t>::max() << endl;
	cout << "Minimum value for offset: " << numeric_limits<offset_t>::min() << endl;
	cout << "Maximum value for offset: " << numeric_limits<offset_t>::max() << endl;
#endif

#ifdef DIRENT
	// Test  a file
	cout << "TESTING DIRENT test.cpp" << endl;
	dirent *finfo = new dirent("test.cpp");
	cout << finfo->print() << endl;

	cout << "TESTING BUFFERED DIRENT OF ABOVE test.cpp" << endl;
	dirent *nfinfo = new dirent(finfo->payload(), finfo->paylen());
	cout << nfinfo->print() << endl;

	// Test a directory
	cout << "TESTING DIRECTORY ENTRY OF ." << endl;
	dirent *dinfo = new dirent(".");
	cout << dinfo->print() << endl;

	cout << "TESTING BUFFERED DIRENT OF ABOVE ." << endl;
	dirent *ndinfo = new dirent(dinfo->payload(), dinfo->paylen());
	cout << ndinfo->print() << endl;

	// Test invalid file
	cout << "TESTING NON EXISTANT FILE frednurke" << endl;
	dirent *iinfo = new dirent("frednurke");
	cout << iinfo->print() << endl;

	cout << "TESTING BUFFERED DIRENT OF ABOVE frednurke" << endl;
	dirent *idinfo = new dirent(iinfo->payload(), iinfo->paylen());
	cout << idinfo->print() << endl;
#endif // DIRENT


#ifdef METADATA
	//Test metadata's

	cout << "m1: F_DESC_16 F_TRANSFER_FILE F_PROGRESS_INPROG 55 test.cpp" << endl;
	metadata* m1 = new metadata( F_DESCRIPTOR_16,
		F_TRANSFER_FILE,
		F_PROGRESS_INPROG,
		55,
		"test.cpp");
	cout << m1->print() << endl << endl << endl;

	cout << "m2: F_DESC_32  F_TRANSFER_FILE F_CSUM_MD5 F_PROGRESS_INPROG 56 test.cpp" << endl;
	metadata* m2 = new metadata( F_DESCRIPTOR_32,
			F_TRANSFER_FILE,
			F_PROGRESS_INPROG,
			56,
			".");
	cout << m2->print() << endl << endl;
	metadata* m3 = new metadata( F_DESCRIPTOR_32,
			F_TRANSFER_FILE,
			F_PROGRESS_INPROG,
			F_CSUM_CRC32,
			56,
			"test");
	cout << m3->print() << endl << endl;
	metadata* m4 = new metadata( F_DESCRIPTOR_32,
			F_TRANSFER_FILE,
			F_PROGRESS_INPROG,
			F_CSUM_MD5,
			56,
			"test");
	cout << m4->print() << endl << endl;

	metadata* m5 = new metadata( F_DESCRIPTOR_32,
			F_TRANSFER_FILE,
			F_PROGRESS_INPROG,
			F_CSUM_SHA1,
			56,
			"test");
	cout << m5->print() << endl << endl;

	metadata* m6 = new metadata(m5->payload(), m5->paylen());

	cout << m6->print() << endl << endl;

#endif // METADATA

#ifdef HOLES

	// offset_t b = numeric_limits<offset_t>::max();

	hole	h(0, 2);
	cout << "h=" << h.print() << endl;
	hole	h2(h);
	cout << "h2=" << h2.print() << endl;
	hole newh = h;
	cout << "Assign h=" << h.print() << " newh=" << newh.print() << endl;
	if (h == h2)
		cout << "h==h2 Correct" << endl;
	else
		cout << "ERROR h==h2" << endl;
	hole h3(4, 5);
	if (h != h3)
		cout << "h!=h3" << endl;
	else
		cout << "ERROR h!=h3" << endl;

	holes	hlist;
	cout << "\n\nBefore:" << hlist.print() << endl;
	hlist.add(10, 8);
	cout << "\n\nAfter:" << hlist.print() << endl;

	hlist.add(20, 8);
	hlist.add(30, 8);
	cout << "\n\nAfter:" << hlist.print() << endl;

	hole h1(21,4);
	holes hl1;
	hl1 += h1;
	hl1.add(29, 5);
	cout << "\n\nAfter:" << hlist.print() << endl;
	hlist -= hl1;
	cout << "\n\nAfter Remove a hole:" << hlist.print() << endl;
	hlist += h3;
	cout << "\n\nAfter Add hole:" << hlist.print() << endl;

	hlist.remove(21, 4);
	cout << "\n\nAfter Remove hole:" << hlist.print() << endl;
	hlist -= h3;
	cout << "\n\nAfter:" << hlist.print() << endl;


	hlist.zap();

	cout << "\n\nAfter Zap:" << hlist.print() << endl;
#endif // HOLES

// #ifdef IPV4TEST
	sarnet::ip ip1("192.94.244.11");
	sarnet::ip ip2("192.94.244.12");
	cout << "IP 1 Address:" << ip1.print() << " IP 2 Address:" << ip2.print() << endl;
	sarnet::ip ip3 = ip2;
	cout << "IP 3 Address:" << ip3.print() << endl;
	if (ip1 == ip2)
		cout << "IP1==IP2 " << ip1.print() << " " << ip2.print() << endl;
	else
		cout << "IP1!=IP2 " << ip1.print() << " " << ip2.print() << endl;
	// sarnet::ip ip2 = ip1; 
	// cout << "IP 2 Address:" << "192.94.244.11" << " " << ip2.str() << endl;
	sarnet::udp udp1("1.2.3.5", 16);
	cout << udp1.print() << endl;
	sarnet::udp udp2("127.0.0.1", 9999);
	cout << udp2.print() << endl;

	string s = "hello world!";
	char *cstr = (char *) malloc(s.length()) + 1;
	strcpy(cstr, s.c_str());
	udp1.tx(cstr, strlen(cstr));
	udp2.tx(cstr, strlen(cstr));

	sarnet::udp inv4(AF_INET, SARATOGASOCKET);
// #endif

//	sarnet::udp udp61("fe80::20c:29ff:fe84:78ab", SARATOGASOCKET);
	sarnet::udp udp61("::1", SARATOGASOCKET);
	cout << udp61.print() << endl;
	udp61.tx(cstr, strlen(cstr));

	char buf[1024];
	ssize_t nread;

	nread = inv4.rx(buf);
	if (nread < 0)
		cout << "ERROR NREAD < 0" << endl;

	sarnet::udp inv6(AF_INET6, SARATOGASOCKET);
	nread = inv6.rx(buf);
	if (nread < 0)
		cout << "ERROR NREAD < 0" << endl;
	sarnet::udp udp62(AF_INET6, SARATOGASOCKET);

	sysinfo df("/dev/sda1");

	uint64_t xfree = df.diskfree();
	cout << "Disk Free is:" << xfree << "kB" << endl;
}

/*
 * $Log: test1.cpp,v $
 * Revision 1.2  2015/01/09 22:09:49  charlesetsmith
 * Moved all Screen output to use curses. There are two windows, the top is status
 * and error messages. THe bottom is command line interaction. Also created
 * separate name spaces for curses (sarwin) and ip (sarip) so they can be
 * independant of the base saratoga code set. Fixed error in SConstruct
 * so that checksums now compiles properly and the checksums SConstruct
 * g++ options are same as the base code set. Huge number of changes but
 * the code set now operational. Remeber you have to "sudo ./saratoga"
 * as multicast requires you to be root to run.
 *
 * Revision 1.1.1.1  2014/10/15 11:55:35  lloydwood
 * Creating saratoga-vallona-dev.
 *
 * This is the Saratoga transfer protocol Vallona implementation,
 * under development by Charles Smith.
 *
 * Revision 1.24  2013/10/07 05:08:11  chas
 * Added status frame formatting. Well first pass code for it. We have a class
 * for status and a class for holes. No where near completed. Just a marker.
 *
 * Revision 1.23  2013/10/04 02:04:18  chas
 * Finally fixed the damn issues with METADATA. Problem was that I was not
 * updating the Frame copy of the metadata. i.e. I needed to copy
 * to _payload and _paylen in the creator for metadata.
 *
 * Revision 1.22  2013/09/29 05:26:28  chas
 * Changed append to s+=
 *
 * Revision 1.21  2013/09/28 20:19:03  chas
 * Moved directory entry flags to separate include file
 *
 * Revision 1.20  2013/09/27 06:15:21  chas
 * Still looking for the bus in metadata. It has something to do with the checksums! I now have crc32 and no checksums working but md5 and sha1 crash me still.
 *
 * Revision 1.19  2013/06/08 07:31:50  chas
 * Direcory entry as Gordon Ramsey would say DONE!! So it can read directory entries and get the file type and info to be ready to go into a metadata or read from to get the info.
 *
 * Revision 1.18  2013/06/07 11:49:14  chas
 * Lots if work done on dirent. Still more work to do. It currently core dumps when you put in an invalid file name. Converting a payload back into a direct isn;t working yet either.
 *
 * Revision 1.17  2013/06/06 23:47:04  chas
 * Sorted out calculations of the available freespace so beacon id now TOTALLY DONE. Tidied up the allocator accordingly so now you only have to pass a pointer to the freespace and it sets the freespace descriptor up accordingly. So you can create beacons without freesoace or with in the allocator.
 *
 * Revision 1.16  2013/06/05 07:20:09  chas
 * Some issues with geting the descriptor size. It is now called descriptor.length(). Also conversion hnton and ntoh issues. Is all consistant now. Beacons good, data good. Lets move on to metadata.
 *
 * Revision 1.15  2013/06/05 01:44:22  chas
 * Sorted out frame's
 *
 * Revision 1.14  2013/06/01 07:32:40  chas
 * Some issues with frame virtualisation. Perhaps this was a bad idea pulling the flags into the frame class... Ill think it over with a couple of scotches (single malt of course).
 *
 * Revision 1.13  2013/05/30 12:43:41  chas
 * LOts of changes to get timestamps going with having Y2K timestamps and
 * working on directory entry records
 *
 * Revision 1.12  2013/03/30 06:17:07  chas
 * Major fixup of flags. We now have the number of bits for each flag
 * and the most significant bit. This along wth the formula
 * 32 - bits - msb gives us the correct shift to do
 *
 * Revision 1.11  2013/03/29 14:05:54  chas
 * Lots of changes to checksums so that we now have a master
 * class "checksum" to deal with in metadata frames.
 * Metadata is moving along. Next big step is getting the
 * dirent class sorted. Time to go to bed its 1:00AM
 *
 * Revision 1.10  2013/03/29 04:25:49  chas
 * Data is done. Work needed on timestamps and flags and some tiding
 * up with beacons creating a descriptor.size() which helps a bit
 *
 * Revision 1.9  2013/03/28 08:56:35  chas
 * Moe bugs in the flags for data. Im shifting the worng number of bits.
 * Think this will be a generic problem to sort as we go through the flags
 * for each frame type.
 *
 * Revision 1.8  2013/03/28 08:22:37  chas
 * Found an error in flags. Some wrong bit shifting for data flags
 * Progressing along with data class. And making printf's a bit more
 * portable with PRIuXX's for _session and _offset types
 *
 * Revision 1.7  2013/03/27 11:09:33  chas
 * Working on Data polymorph frame type now. Setting and getting flags
 * applicable to data and the optional timestamp. Some more thinking
 * on how I stamp the timestamp. THink it is best done within the
 * class constructor with an optional data.timestamp() to update it to
 * now when needed.
 *
 * Revision 1.6  2013/03/24 22:40:32  chas
 * Added flag changes to beacon. Will need to add more for other flags
 * and add the appropriate functions to other frame types as well.
 * I'll set up this in frame as well as virtual function calls.
 * This will ensure that only the appropriate flags are set for the approriate frame type.
 *
 * Revision 1.5  2013/03/24 22:00:26  chas
 * Added data.h and split out beacon.h from frame.h
 *
 * Revision 1.4  2013/03/23 12:01:00  chas
 * Finally sorted out polymorhic "frame" class and beacon all done
 * so now I can move onto similar for other frame types
 *
 * Revision 1.3  2013/03/15 12:17:34  chas
 * Moving along with the polymorphism. Beacon nearly done
 *
 * Revision 1.2  2013/03/15 04:55:42  chas
 * Moving frames to polymophism so we have a single base class of frame and
 * then polymorph frame types of beacon and data. As we move forward
 * I will add the other saratoga frame types. Lots more work to do here yet
 * as I havn't got the base class sorted yet.
 *
 * Revision 1.1.1.1  2013/03/09 07:35:04  chas
 * Saratoga C++ Primary Implementation
 * Charles Smith charlesetsmith@me.com
 *
 */
