/*
 * $Id: tran.h,v 1.2 2015/01/09 22:09:49 charlesetsmith Exp $
 */

/*

 Copyright (c) 2011, Charles Smith
 All rights reserved.

 Redistribution and use in source and binary forms, with or without modification,
 are permitted provided that the following conditions are met:

    * Redistributions of source code must retain the above copyright notice, this
      list of conditions and the following disclaimer.
    * Redistributions in binary form must reproduce the above copyright notice, this
      list of conditions and the following disclaimer in the documentation and/or
      other materials provided with the distribution.
    * Neither the name of Vallona Networks nor the names of its contributors
      may be used to endorse or promote products derived from this software without
      specific prior written permission.

 THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND
 ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
 IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT,
 INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING,
 BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
 LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE
 OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED
 OF THE POSSIBILITY OF SUCH DAMAGE.

 */

#ifndef _TRAN_H
#define _TRAN_H

#include <iostream>
#include <cstring>
#include <string>
using namespace std;

#include "saratoga.h"
#include "sarflags.h"
#include "screen.h"
#include "timestamp.h"
#include "frame.h"
#include "holes.h"

namespace saratoga
{
/*
 **********************************************************************
 * TRAN
 **********************************************************************
 */

enum txorrx
{
	TO_PEER = 0x00,		// I am sending TO another peer
	FROM_PEER = 0x01	// I am receiving FROM another peer
};

/*
 * Data pertaining to each saratoga transaction
 */
class tran
{
private:
        session_t		_session;	// The session ID
	enum txorrx		_txorrx;	// Sending TO or Receving FROM a socket
	holes			_holes;		// Current list of holes
	udp			_peer;		// Socket I am talking to
	fileio			_local;		// Local file I am reading or writing to
	uint64_t		_diskfree;	// Current free disk space in kilobytes
	string			_fname;		// Local file name

// THESE ARE ALL OF THE FLAGS APPLICABLE TO THE TRANSACTION	
	Fversion	_version(F_VERSION_1);	// Saratoga Version 1 default
	Fdescriptor	_descriptor(F_DESCRIPTOR_16);	// Descriptor size
	Ftransfer	_transfer(F_TRANSFER_FILE);	// What are we transferring 
	Ftsamp		_tstamp(F_TIMESTAMP_YES);	// Do we transfer time stamps
	Fprogress	_progress(F_PROGRESS_INPROG);	// Are we inprogress or terminated
	Ftxwilling	_txwilling(F_TXWILLING_YES);	// Can we transmit at the moment
	Frxwilling	_rxwilling(F_RXWILLING_YES);	// Can we receive at the moment
	Fudptype	_udptype(F_UDPONLY);	// Are we udp or udplite
	F_metadatarecvd	_metadatarecvd(F_METADATARECVD_NO);	// Has Metadata been received
	Fsessiontype	_sessiontype(F_SESSIONTYPE_GETFILEORDIR);	// Get file/dir or deletefile
	Ffileordir	_fileordir(F_FILEORDIR_FILE;	// Are we a file or directory
	Fholes		_holesreq(F_HOLES_SENTVOLUNTARILY);	// Are holes requested or 
	Freqstatus	_reqstatus(F_REQSTATUS_YES);	// SHould we request a staus
	Feod		_eod(F_EOD_NO);		// End of data received	
	Ffreespace	_freespace(F_FREESPACE_YES);	// Do we advertise freespace
	Ffreespaced	_freespaced(F_FREESPACED_16);	// What is the freespace descriptor
	Fcsum		_csum(F_CSUM_NONE);		// Checksums
	Fcsumlen	_csumlen(F_CSUMLEN_NONE);		// Checksum type
	Ferrcode	_errcode(F_ERRCODE_SUCCESS);	// Current error code
public:
	tran()	{ };
	~tran()	{ };
	// Are we the trnsmitter or receiver ?
	enum txorrx	direction() { return _txorrx; };


	// Set and get all the flags applicable to a transaction
	enum f_version	version() { return(_version.get()); };
	void version(f_version i) { i = F_VERSION_1; _version.set(i); };
	enum f_descriptor descriptor() { return(_descriptor.get()); };
	void decriptor(f_decriptor i) { _decriptor.set(i); };
	enum f_transfer transfer() { return(_transfer.get()); };
	void transfer(f_transfer i) { _transfer.set(i); };
	enum f_tstamp tstamp() { return(_tstamp.get()); };
	void tstamp(f_tstamp i) { _tstamp.set(i); };
	enum f_progress progress() { return(_progress.get()); };
	void progress(f_progress i) { _progress.set(i); };
	enum f_txwilling txwilling() { return(_txwilling.get()); };
	void txwilling(f_txwilling i) { _txwilling.set(i); };
	enum f_rxwilling rxwilling() { return(_rxwilling.get()); };
	void rxwilling(f_rxwilling i) { _rxwilling.set(i); };
	enum f_udptype udptype() { return(_udptype.get()); };
	void udptype(f_udptype i) { _udptype.set(i); };
	enum f_metadatarecvd metadatarecvd() { return(_metadatarecvd.get()); };
	void metadatarecvd(f_metadatarecvd i) { _metadatarecvd.set(i); };
	enum f_sessiontype sessiontype() { return(_sessiontype.get()); };
	void sessiontype(f_sessiontype i) { _sessiontype.set(i); };
	enum f_fileordir fileordir() { return(_fileordir.get()); };
	void fileordir(f_fileordir i) { _fileordir.set(i); };
	enum f_holesreq holesreq() { return(_holesreq.get()); };
	void holesreq(f_holesreq i) { _holesreq.set(i); };
	enum f_reqstatus reqstatus() { return(_reqstatus.get()); };
	void reqstatus(f_reqstatus i) { _reqstatus.set(i); };
	enum f_eod eod() { return(_eod.get()); };
	void eod(f_eod i) { _eod.set(i); };
	enum f_freespace freespace() { return(_freespace.get()); };
	void freespace(f_freespace i) { _freespace.set(i); };
	enum f_freespaced freespaced() { return(_freespaced.get()); };
	void freespaced(f_freespaced i) { _freespaced.set(i); };
	enum f_csum csum() { return(_csum.get()); };
	void csum(f_csum i) { _csum.set(i); };
	enum f_cseumlen cseumlen() { return(_cseumlen.get()); };
	void cseumlen(f_cseumlen i) { _cseumlen.set(i); };
	enum f_errcode errcode() { return(_errcode.get()); };
	void errcode(f_errcode i) { _errcode.set(i); };
};

} // Namespace saratoga

#endif // _TRAN_H

/*
 * $Log: tran.h,v $
 * Revision 1.2  2015/01/09 22:09:49  charlesetsmith
 * Moved all Screen output to use curses. There are two windows, the top is status
 * and error messages. THe bottom is command line interaction. Also created
 * separate name spaces for curses (sarwin) and ip (sarip) so they can be
 * independant of the base saratoga code set. Fixed error in SConstruct
 * so that checksums now compiles properly and the checksums SConstruct
 * g++ options are same as the base code set. Huge number of changes but
 * the code set now operational. Remeber you have to "sudo ./saratoga"
 * as multicast requires you to be root to run.
 *
 * Revision 1.1.1.1  2014/10/15 11:55:35  lloydwood
 * Creating saratoga-vallona-dev.
 *
 * This is the Saratoga transfer protocol Vallona implementation,
 * under development by Charles Smith.
 *
 */

